
package data.scripts;

import com.fs.starfarer.api.AnimationAPI;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.EveryFrameWeaponEffectPlugin;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.WeaponAPI;
/**
 * FROM MESOTRONIK
 */
public class CitadelBurstFireMinigunScript implements EveryFrameWeaponEffectPlugin {


    private static final float ANIM_FRAMERATE = 20f;
    private static final float ANIM_MIN_FRAMERATE = 14f;
    private static final float ANIM_DURATION = 0.8f;
    private static final float ANIM_COOLDOWN_POINT = 2f;
    private boolean firing = false;
    private int curFrame = 0;
    private float fireTimer = 999f, timeSinceLastFrame = 0f;

    private float calcFramerate()
    {
        // Full FPS while barrels are firing
        if (fireTimer <= ANIM_COOLDOWN_POINT)
        {
            return ANIM_FRAMERATE;
        }

        // Animation slows down from 1/2 second onward during cooldown
        final float speedMod = 1f - ((fireTimer - ANIM_COOLDOWN_POINT)
                / (ANIM_DURATION - ANIM_COOLDOWN_POINT));
        return Math.max(ANIM_MIN_FRAMERATE, ANIM_FRAMERATE * speedMod);
    }

    private void advanceAnimation(AnimationAPI anim, float amount)
    {
        final float timeBetweenFrames = 1f / calcFramerate();
        fireTimer += amount;
        timeSinceLastFrame += amount;
        while (timeSinceLastFrame >= timeBetweenFrames)
        {
            timeSinceLastFrame -= timeBetweenFrames;
            if (++curFrame >= anim.getNumFrames())
            {
                curFrame = 0;
            }
        }
    }

    @Override
    public void advance(float amount, CombatEngineAPI engine, WeaponAPI weapon)
    {
        if (engine.isPaused() || weapon.getSlot().isHidden())
        {
            return;
        }

        AnimationAPI anim = weapon.getAnimation();
        ShipAPI ship = weapon.getShip();

        // On death, instantly set to frame 0
        if (ship.isHulk())
        {
            anim.setFrame(0);
            return;
        }

        if (firing)
        {
            // Detect when burst is complete
            if (weapon.getCooldownRemaining() <= 0f && weapon.getChargeLevel() < 1f)
            {
                firing = false;
            }
        }
                else
        {
            // This is a new burst, play sound and start animation
            if (weapon.getChargeLevel() >= 1f)
            {
                firing = true;
                fireTimer = 0f;
                timeSinceLastFrame = amount;
            }
        }

        // Advance animation if timer has started, coast to frame 0 afterwards
        if (fireTimer < ANIM_DURATION || curFrame > 0)
        {
            advanceAnimation(anim, amount);
        }

        // Frame must be manually set every frame to overwrite vanilla animation
        anim.setFrame(curFrame);
    }
}

