package data.shipsystems.scripts;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.FluxTrackerAPI;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.plugins.ShipSystemStatsScript;

public class FluxDissipatorStats implements ShipSystemStatsScript {

    private static final float SHIELD_ARC_BONUS = 360f;
    private static final float SHIELD_FLUX_PER_SECOND = 0.02f;
    private float currentHardFlux;
    private float oldShieldArc;
    private boolean started = false;

    @Override
    public void apply(MutableShipStatsAPI stats, String id, State state, float effectLevel) {
        ShipAPI ship = (ShipAPI) stats.getEntity();
        if (ship == null || ship.getShield() == null) {
            return;
        }

        FluxTrackerAPI flux = ship.getFluxTracker();

        if (started == false) {
            oldShieldArc = ship.getShield().getArc();
            currentHardFlux = flux.getHardFlux();
            ship.getShield().setArc(SHIELD_ARC_BONUS);
            ship.getShield().setActiveArc(SHIELD_ARC_BONUS);
            started = true;
        }

        stats.getShieldDamageTakenMult().modifyMult(id, 1f - (0.1f * effectLevel));
        stats.getShieldUpkeepMult().modifyMult(id, 0.0f * effectLevel);
        stats.getShieldUnfoldRateMult().modifyPercent(id, 2000f * effectLevel);
        
        currentHardFlux += Global.getCombatEngine().getElapsedInLastFrame() * SHIELD_FLUX_PER_SECOND * ship.getMutableStats().getFluxCapacity().getBaseValue();

        if (state != State.OUT) {
            flux.setHardFlux(currentHardFlux);
        } else {
            currentHardFlux = flux.getHardFlux();
        }
                
    }

    @Override
    public StatusData getStatusData(int index, State state, float effectLevel) {
        if (index == 0) {
            return new StatusData("overcharging shield generator", false);
        }
        return null;
    }

    @Override
    public void unapply(MutableShipStatsAPI stats, String id) {
        ShipAPI ship = (ShipAPI) stats.getEntity();
        if (ship == null || ship.getShield() == null) {
            return;
        }

        if (started == true) {
            ship.getShield().setArc(oldShieldArc);
            ship.getShield().setActiveArc(oldShieldArc);
            started = false;
        }

        stats.getShieldDamageTakenMult().unmodify(id);
        stats.getShieldUpkeepMult().unmodify(id);
        stats.getShieldUnfoldRateMult().unmodify(id);
    }
}