package data.shipsystems.scripts.ai;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.CombatAssignmentType;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.CombatFleetManagerAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipSystemAIScript;
import com.fs.starfarer.api.combat.ShipSystemAPI;
import com.fs.starfarer.api.combat.ShipwideAIFlags;
import com.fs.starfarer.api.combat.ShipwideAIFlags.AIFlags;
import com.fs.starfarer.api.util.IntervalUtil;
import org.lazywizard.lazylib.MathUtils;
import org.lwjgl.util.vector.Vector2f;

public class PowerCruiseAI implements ShipSystemAIScript {

    private ShipAPI ship;
    private ShipwideAIFlags flags;

    private final IntervalUtil tracker = new IntervalUtil(0.75f, 1.5f);

    @Override
    public void init(ShipAPI ship, ShipSystemAPI system, ShipwideAIFlags flags, CombatEngineAPI engine) {
        this.ship = ship;
        this.flags = flags;
    }

    @Override
    public void advance(float amount, Vector2f missileDangerDir, Vector2f collisionDangerDir, ShipAPI target) {
        if (Global.getCombatEngine().isPaused()) {
            return;
        }

        tracker.advance(amount);

        if (tracker.intervalElapsed()) {
            if (ship.getFluxTracker().isOverloadedOrVenting()) {
                return;
            }

            float decisionLevel = 0f;
            
            CombatFleetManagerAPI.AssignmentInfo assignment = Global.getCombatEngine().getFleetManager(ship.getOwner()).getTaskManager(true).getAssignmentFor(ship);
            Vector2f targetSpot;
            if (assignment != null && assignment.getTarget() != null) {
                if (!ship.getSystem().isOn()) {
                    float distance = MathUtils.getDistance(ship, assignment.getTarget().getLocation());

                    if (distance > 3000f && assignment.getType() == CombatAssignmentType.ASSAULT) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 2000f && assignment.getType() == CombatAssignmentType.CAPTURE) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 3000f && assignment.getType() == CombatAssignmentType.CONTROL) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 3000f && assignment.getType() == CombatAssignmentType.ENGAGE) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 3000f && assignment.getType() == CombatAssignmentType.HARASS) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 3000f && (assignment.getType() == CombatAssignmentType.HEAVY_ESCORT || assignment.getType() == CombatAssignmentType.LIGHT_ESCORT || assignment.getType() == CombatAssignmentType.MEDIUM_ESCORT)) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 2000f && assignment.getType() == CombatAssignmentType.INTERCEPT) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 2500f && assignment.getType() == CombatAssignmentType.RALLY_TASK_FORCE) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 2500f && assignment.getType() == CombatAssignmentType.RECON) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else {
                        targetSpot = null;
                    }
                } else {
                    float distance = MathUtils.getDistance(ship, assignment.getTarget().getLocation());

                    if (distance > 500f && assignment.getType() == CombatAssignmentType.ASSAULT) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 500f && assignment.getType() == CombatAssignmentType.CAPTURE) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 500f && assignment.getType() == CombatAssignmentType.CONTROL) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 1000f && assignment.getType() == CombatAssignmentType.ENGAGE) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 1000f && assignment.getType() == CombatAssignmentType.HARASS) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 1000f && (assignment.getType() == CombatAssignmentType.HEAVY_ESCORT || assignment.getType() == CombatAssignmentType.LIGHT_ESCORT || assignment.getType() == CombatAssignmentType.MEDIUM_ESCORT)) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 500f && assignment.getType() == CombatAssignmentType.INTERCEPT) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 1000f && assignment.getType() == CombatAssignmentType.RALLY_TASK_FORCE) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else if (distance > 500f && assignment.getType() == CombatAssignmentType.RECON) {
                        targetSpot = assignment.getTarget().getLocation();
                    } else {
                        targetSpot = null;
                    }
                }
            } else {
                targetSpot = null;
            }

            if (flags.hasFlag(AIFlags.RUN_QUICKLY)) {
                decisionLevel += 10f;
            } else if (flags.hasFlag(AIFlags.PURSUING)) {
                decisionLevel += 10f;
            } else if (targetSpot != null) {
                decisionLevel += 12.5f;
            }
            if (flags.hasFlag(AIFlags.TURN_QUICKLY)) {
                decisionLevel *= 1.25f;
            }
            if (flags.hasFlag(AIFlags.BACK_OFF)) {
                decisionLevel *= 0.5f;
            }
            if (flags.hasFlag(AIFlags.NEEDS_HELP)) {
                decisionLevel *= 0.75f;
            }
            if (flags.hasFlag(AIFlags.HAS_INCOMING_DAMAGE)) {
                decisionLevel *= 0.75f;
            }

            if (decisionLevel >= 7f ^ ship.getSystem().isOn()) {
                ship.useSystem();
            }
        }
    }
}
