package com.fs.starfarer.api.campaign;

import java.awt.Color;
import java.util.List;
import java.util.Map;

import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.EveryFrameScript;
import com.fs.starfarer.api.campaign.CampaignEventListener.FleetDespawnReason;
import com.fs.starfarer.api.campaign.JumpPointAPI.JumpDestination;
import com.fs.starfarer.api.campaign.ReputationActionResponsePlugin.ReputationAdjustmentResult;
import com.fs.starfarer.api.campaign.comm.MessagePriority;
import com.fs.starfarer.api.campaign.econ.EconomyAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.events.CampaignEventManagerAPI;
import com.fs.starfarer.api.campaign.events.CampaignEventPlugin;
import com.fs.starfarer.api.campaign.events.ReportAPI;
import com.fs.starfarer.api.campaign.rules.MemoryAPI;
import com.fs.starfarer.api.campaign.rules.RulesAPI;
import com.fs.starfarer.api.characters.AbilityPlugin;
import com.fs.starfarer.api.characters.ImportantPeopleAPI;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.combat.EngagementResultAPI;
import com.fs.starfarer.api.combat.ViewportAPI;


/**
 * Note: generics can not be used in scripts.
 * They are used in this API purely to show the return/parameter types of functions more clearly.
 * @author Alex Mosolov
 *
 * Copyright 2012 Fractal Softworks, LLC
 */
@SuppressWarnings("unchecked")
public interface SectorAPI {
	
	/**
	 * Can be used (including by classes that are not part of the savegame - i.e. ModPlugin or 
	 * transient CampaignPlugin implementations) to save data between sessions.
	 * @return
	 */
	Map<String, Object> getPersistentData();
	
	void registerPlugin(CampaignPlugin plugin);
	void unregisterPlugin(String pluginId);
	
	StarSystemAPI getStarSystem(String name);
	StarSystemAPI createStarSystem(String name);
	List<StarSystemAPI> getStarSystems();
	void removeStarSystem(StarSystemAPI system);
	void setCurrentLocation(LocationAPI location);
	
	LocationAPI getHyperspace();
	
	/**
	 * @param fleet
	 * @param jumpLocation can be null. If not, fleet will attempt to reach it before jumping. Failure will result in aborted jump.
	 * @param dest
	 */
	void doHyperspaceTransition(CampaignFleetAPI fleet, SectorEntityToken jumpLocation, JumpDestination dest);
	
	
	CampaignFleetAPI createFleet(String factionId, String fleetTypeId);
	CampaignClockAPI getClock();
	
	CampaignFleetAPI getPlayerFleet();
	
	FactionAPI getFaction(String factionId);
	List<FactionAPI> getAllFactions();
	
	List<String> getAllWeaponIds();
	List<String> getAllEmptyVariantIds();
	List<String> getAllFighterWingIds();
	
	
	CampaignUIAPI getCampaignUI();
	
	CampaignEventManagerAPI getEventManager();
	
	void setPaused(boolean paused);
	boolean isPaused();
	
	void addScript(EveryFrameScript script);
	void removeScriptsOfClass(Class c);
	void removeScript(EveryFrameScript script);
	
	void addTransientScript(EveryFrameScript script);
	void removeTransientScript(EveryFrameScript script);
	
	
	/**
	 * Use SectorAPI.getCampaignUI.addMessage instead.
	 * @param text
	 */
	@Deprecated void addMessage(String text);
	
	/**
	 * Use SectorAPI.getCampaignUI.addMessage instead.
	 * @param text
	 */
	@Deprecated void addMessage(String text, Color color);
	
	/**
	 * Use SectorAPI.getCampaignUI.addMessage instead.
	 * @param text
	 */
	@Deprecated void addMessage(String text, String highlight, Color highlightColor);
	
	/**
	 * Use SectorAPI.getCampaignUI.addMessage instead.
	 * @param text
	 */
	@Deprecated void addMessage(String text, Color color, String highlight, Color highlightColor);


	
	LocationAPI getCurrentLocation();
	
	LocationAPI getRespawnLocation();
	void setRespawnLocation(LocationAPI respawnLocation);
	Vector2f getRespawnCoordinates();
	
	
	CharacterDataAPI getCharacterData();
	
	
	void reportEventStage(CampaignEventPlugin event, String stage, MessagePriority priority);
	void reportEventStage(CampaignEventPlugin event, String stage, SectorEntityToken sendFrom, MessagePriority priority);
	
	/**
	 * When sendFrom is the player fleet, the message will be delivered immediately, regardless of
	 * whether there's a comm relay nearby, and its "deliveredBy" will be set to "internal network".
	 * The default message image will still be based on the event target, in this case.
	 *
	 * If sendFrom is *not* the player fleet, the default message image will be based on the sendFrom's
	 * owner faction.
	 * 
	 * "deliveredBy" will also be set to "internal network" if the report generated by this method is
	 * delivered via a player intel channel, though in this case it will take a normal amount of time
	 * to send and still requires a comm relay. (Ostensibly, the time simulates the time it takes to
	 * get the intel to base the report on.)
	 * 
	 * @param event
	 * @param stage
	 * @param sendFrom
	 * @param priority
	 * @param onDelivery
	 */
	void reportEventStage(CampaignEventPlugin event, String stage, SectorEntityToken sendFrom, MessagePriority priority, OnMessageDeliveryScript onDelivery);
	
	RulesAPI getRules();
	
	
	long getLastPlayerBattleTimestamp();
	boolean isLastPlayerBattleWon();
	
	/**
	 * "global" memory.
	 * @return
	 */
	MemoryAPI getMemory();
	MemoryAPI getMemoryWithoutUpdate();
	
	IntelDataAPI getIntel();
	
	
	/**
	 * Will look for the entity in all LocationAPIs.
	 * @param id
	 * @return
	 */
	SectorEntityToken getEntityById(String id);
	

	/**
	 * Iterates through hyperspace and all star systems.
	 * @param tag
	 * @return
	 */
	List<SectorEntityToken> getEntitiesWithTag(String tag);
	
	
	EconomyAPI getEconomy();

	
	
	
	void addListener(CampaignEventListener listener);
	void addTransientListener(CampaignEventListener listener);
	/**
	 * Adds a listener that will be automatically removed unless this method is called again within
	 * the specified number of days.
	 * @param listener
	 * @param daysToKeep
	 */
	void addListenerWithTimeout(CampaignEventListener listener, float daysToKeep);
	
	void removeListener(CampaignEventListener listener);

	/**
	 * Regular, transient, w/ timeout.
	 * @return
	 */
	List<CampaignEventListener> getAllListeners();
	
	void reportPlayerMarketTransaction(PlayerMarketTransaction transaction);
	void reportFleetDespawned(CampaignFleetAPI fleet, FleetDespawnReason reason, Object param);
	
	
	// this gets called automatically when a fleet is added to a location for the first time
	//void reportFleetSpawned(CampaignFleetAPI fleet);
	
	
	void reportFleetJumped(CampaignFleetAPI fleet, SectorEntityToken from, JumpDestination to);
	void reportFleetReachedEntity(CampaignFleetAPI fleet, SectorEntityToken entity);
	
	
	/**
	 * Called once per each autoresolve round of the battle.
	 * Also called after a player battle occurred.
	 * 
	 * Fleets and their compositons snapshotted right before the autoresolve round.
	 * @param battle
	 * @param primaryWinner 
	 */
	void reportBattleOccurred(CampaignFleetAPI primaryWinner, BattleAPI battle);
	
	/**
	 * Called after an autoresolve round that finishes a battle, after reportBattleOccurred.
	 * Also called after a player battle occurred.
	 * 
	 * Fleets and their compositons snapshotted right before the autoresolve round.
	 * @param battle
	 * @param primaryWinner
	 */
	void reportBattleFinished(CampaignFleetAPI primaryWinner, BattleAPI battle);

	
	/**
	 * Adjust the player's reputation with the specified faction, based on the action.
	 * 
	 * This method will pick the highest-priority ReputationActionResponsePlugin and
	 * call its handlePlayerReputationAction() method with these parameters.
	 * 
	 * See CoreCampaignPluginImpl.handlePlayerReputationAction() for an implementation example.
	 * 
	 * @param action
	 * @param faction
	 * @return
	 */
	ReputationAdjustmentResult adjustPlayerReputation(Object action, String factionId);
	
	/**
	 * Adjust the player's reputation with the specified person, based on the action.
	 * 
	 * This method will pick the highest-priority ReputationActionResponsePlugin and
	 * call its handlePlayerReputationAction() method with these parameters.
	 * 
	 * See CoreCampaignPluginImpl.handlePlayerReputationAction() for an implementation example.
	 * 
	 * @param action
	 * @param person
	 * @return
	 */
	ReputationAdjustmentResult adjustPlayerReputation(Object action, PersonAPI person);

	
	/**
	 * See data/campaign/pings.json for ping types.
	 * @param entity
	 * @param pingType
	 */
	void addPing(SectorEntityToken entity, String pingType);

	
	/**
	 * Shouldn't have to call this as the relevant showInteractionDialog() methods already do.
	 * @param dialog
	 */
	void reportShowInteractionDialog(InteractionDialogAPI dialog);

	
	/**
	 * Player opened the core UI while interacting with an entity that has a market.
	 * @param market
	 */
	void reportPlayerOpenedMarket(MarketAPI market);

	
	void reportPlayerReputationChange(String faction, float delta);
	void reportPlayerReputationChange(PersonAPI person, float delta);
	
	void reportPlayerEngagement(EngagementResultAPI result);
	
	/**
	 * true during the during-game-creation time passing.
	 * @return
	 */
	boolean isInNewGameAdvance();

	PersonAPI getPlayerPerson();

	
	void reportPlayerOpenedMarketAndCargoUpdated(MarketAPI market);
	void reportEncounterLootGenerated(FleetEncounterContextPlugin plugin, CargoAPI loot);
	void reportPlayerClosedMarket(MarketAPI market);

	void reportPlayerActivatedAbility(AbilityPlugin ability, Object param);
	void reportPlayerDeactivatedAbility(AbilityPlugin ability, Object param);
	
	MissionBoardAPI getMissionBoard();

	
	ReportAPI getReport(String eventType, String eventStage, String channelId);

	ImportantPeopleAPI getImportantPeople();

	FactionAPI getPlayerFaction();

	void addPing(SectorEntityToken entity, String pingType, Color colorOverride);

	ViewportAPI getViewport();

	/**
	 * All star systems and hyperspace.
	 * @return
	 */
	List<LocationAPI> getAllLocations();

	String getDifficulty();
	void setDifficulty(String difficulty);

	boolean isIronMode();


}






