package com.fs.starfarer.api.campaign.econ;

import com.fs.starfarer.api.combat.MutableStat;
import com.fs.starfarer.api.combat.StatBonus;

/**
 * 
 * There's no expectation of the various stockpile-related values being exact.
 * The econ simulation is based on rolling averages "working out" after enough time.
 * 
 * In particular, the (non-average) stockpile value is highly volatile, and so
 * isn't used for things like determining how much of a particular commodity is available
 * for sale at a market. The average (after demand) is used instead.
 * 
 * @author Alex Mosolov
 *
 * Copyright 2014 Fractal Softworks, LLC
 */
public interface CommodityOnMarketAPI {

	String getId();
	
	MutableStat getSupply();
	MutableStat getGreed();
	
	float getSupplyValue();
	float getGreedValue();
	
	CommoditySpecAPI getCommodity();

	
	float getStockpile();
	void setStockpile(float stockpile);
	
	void addToStockpile(float quantity);
	void removeFromStockpile(float quantity);

	MarketAPI getMarket();

	float getAverageStockpile();
	void setAverageStockpile(float averageStockpile);
	
	float getAverageStockpileAfterDemand();
	void setAverageStockpileAfterDemand(float averageStockpileAfterDemand);

	String getDemandClass();
	MarketDemandAPI getDemand();

	float getUtilityOnMarket();

	boolean isPersonnel();
	boolean isFuel();

	void addToAverageStockpile(float quantity);
	void removeFromAverageStockpile(float quantity);

	StatBonus getPlayerPriceMod();

	TransactionTrackerAPI getTransactions();

	float getPlayerTradeStockpileImpactMult();
	
}




