package com.fs.starfarer.api.impl.campaign.abilities;

import java.awt.Color;

import com.fs.starfarer.api.EveryFrameScript;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.BattleAPI;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.characters.AbilityPlugin;
import com.fs.starfarer.api.loading.AbilitySpecAPI;
import com.fs.starfarer.api.ui.TooltipMakerAPI;

public class BaseAbilityPlugin implements AbilityPlugin, EveryFrameScript{
	protected SectorEntityToken entity;
	protected String id;
	
	protected int disableFrames = 0;
	
	protected transient AbilitySpecAPI spec = null;
	
	public void init(String id, SectorEntityToken entity) {
		this.id = id;
		this.entity = entity;
		readResolve();
	}
	
	protected Object readResolve() {
		spec = Global.getSettings().getAbilitySpec(id);
		return this;
	}
	Object writeReplace() {
		return this;
	}
	
	public String getOnSoundUI() { return spec.getUIOn(); }
	public String getOnSoundWorld() { return spec.getWorldOn(); }
	public String getOffSoundUI() { return spec.getUIOff(); }
	public String getOffSoundWorld() { return spec.getWorldOff(); }

	// no support for UI loops currently; just playing a stereo sound is a workaround
	// but doesn't use the dedicated UI sound sources
	public String getLoopSoundUI() { return spec.getUILoop(); }
	public float getLoopSoundUIVolume() { return 1f; }
	public float getLoopSoundUIPitch() { return 1f; }
	
	public String getLoopSoundWorld() { return spec.getWorldLoop(); }
	public float getLoopSoundWorldVolume() { return 1f; }
	public float getLoopSoundWorldPitch() { return 1f; }
	
	
	
	public String getModId() {
		return id + "_ability_mod";
	}
	
	public CampaignFleetAPI getFleet() {
		if (entity instanceof CampaignFleetAPI) {
			return (CampaignFleetAPI) entity;
		}
		return null;
	}
	
	public SectorEntityToken getEntity() {
		return entity;
	}
	
	public String getId() {
		return id;
	}
	
	public void advance(float amount) {
		disableFrames--;
		if (disableFrames < 0) disableFrames = 0;
	}

	public boolean isDone() {
		return false;
	}

	public boolean runWhilePaused() {
		return false;
	}
	
	public boolean showActiveIndicator() {
		return isActive();
	}
	
	public boolean isUsable() {
		return !isOnCooldown() && disableFrames <= 0;
	}
	
	public void forceDisable() {
		disableFrames = 2;
	}
	
	public float getCooldownFraction() {
		return 1f;
	}
	
	public boolean hasCustomButtonPressSounds() {
		return false;
	}
	
	public boolean hasTooltip() {
		return false;
	}
	
	public void createTooltip(TooltipMakerAPI tooltip, boolean expanded) {
	}
	
	public boolean isTooltipExpandable() {
		return false;
	}
	
	public float getTooltipWidth() {
		return 350f;
	}


	public void pressButton() {
		
	}

	public String getSpriteName() {
		return Global.getSettings().getSpriteName("abilities", "empty_slot");
	}

	public void activate() {
		CampaignFleetAPI fleet = getFleet();
		if (fleet == null || !fleet.isPlayerFleet()) return;
		Global.getSector().reportPlayerActivatedAbility(this, null);
	}

	public void deactivate() {
		CampaignFleetAPI fleet = getFleet();
		if (fleet == null || !fleet.isPlayerFleet()) return;
		Global.getSector().reportPlayerDeactivatedAbility(this, null);
	}


	private static Color defaultCooldownColor = new Color(0,0,0,171);
	public Color getCooldownColor() {
		return defaultCooldownColor;
	}

	public Color getProgressColor() {
		return entity.getFaction().getBrightUIColor();
	}
	
	public Color getActiveColor() {
		return entity.getFaction().getBrightUIColor();
	}

	public float getProgressFraction() {
		return 0;
	}

	public boolean isActive() {
		return false;
	}

	public boolean isActiveOrInProgress() {
		return isActive() || isInProgress();
	}

	public boolean isInProgress() {
		return getProgressFraction() > 0;
	}

	public boolean showCooldownIndicator() {
		return getCooldownFraction() < 1;
	}

	public boolean showProgressIndicator() {
		return getProgressFraction() > 0;
	}
	
	
	public boolean isOnCooldown() {
		return getCooldownFraction() < 1f;
	}

	public void cleanup() {
		
	}

	public boolean isCooldownRenderingAdditive() {
		return false;
	}
	
	
	
	protected String getActivationText() {
		return null;
	}
	
	protected String getDeactivationText() {
		return null;
	}

	public void fleetJoinedBattle(BattleAPI battle) {
	}

	public void fleetLeftBattle(BattleAPI battle, boolean engagedInHostilities) {
	}
}
