package com.fs.starfarer.api.impl.campaign.abilities;

import java.awt.Color;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.characters.AbilityPlugin;
import com.fs.starfarer.api.fleet.FleetMemberViewAPI;
import com.fs.starfarer.api.impl.campaign.ids.Abilities;
import com.fs.starfarer.api.ui.LabelAPI;
import com.fs.starfarer.api.ui.TooltipMakerAPI;
import com.fs.starfarer.api.util.Misc;

public class SustainedBurnAbility extends BaseToggleAbility {

	public static final float SENSOR_RANGE_MULT = 0.5f;
	public static final float DETECTABILITY_PERCENT = 100f;
	public static final float MAX_BURN_MOD = 5f;
	public static final float INITIAL_BURN_PENALTY = 0f;
	private static final float ACCELERATION_MULT = 0.25f;
	
	public String getSpriteName() {
		return Global.getSettings().getSpriteName("abilities", Abilities.SUSTAINED_BURN);
	}
	
	
	@Override
	protected String getActivationText() {
		return "Sustained burn";
	}


	@Override
	protected void activateImpl() {
//		if (entity.isInCurrentLocation()) {
//			entity.addFloatingText("Sustained burn", entity.getFaction().getBaseUIColor(), 0.5f);
//		}
		
		AbilityPlugin goDark = entity.getAbility(Abilities.GO_DARK);
		if (goDark != null && goDark.isActive()) {
			goDark.deactivate();
		}
		AbilityPlugin eb = entity.getAbility(Abilities.EMERGENCY_BURN);
		if (eb != null && eb.isActive()) {
			eb.deactivate();
		}
	}

	@Override
	protected void applyEffect(float level) {
		CampaignFleetAPI fleet = getFleet();
		if (fleet == null) return;
		
		//System.out.println("Level: " + level);
		//level = 0.01f;
		
		fleet.getStats().getSensorRangeMod().modifyMult(getModId(), 1f + (SENSOR_RANGE_MULT - 1f) * level, "Sustained burn");
		fleet.getStats().getDetectedRangeMod().modifyPercent(getModId(), DETECTABILITY_PERCENT * level, "Sustained burn");
		
		//int burnModifier = (int)(MAX_BURN_MOD * level) - (int)(INITIAL_BURN_PENALTY * (1f - level));
		//int burnModifier = (int)(MAX_BURN_MOD * level);
		int burnModifier = 0;
		float burnMult = 1f; 
		if (level > 0.5f) {
			burnModifier = (int)(MAX_BURN_MOD * (level - 0.5f) / 0.5f);
		} else {
			//burnModifier = -1 * (int)(INITIAL_BURN_PENALTY * (1f - level / 0.5f));
			burnMult = 1f + ((INITIAL_BURN_PENALTY - 1f) * (1f - level / 0.5f));
		}
		fleet.getStats().getFleetwideMaxBurnMod().modifyFlat(getModId(), burnModifier, "Sustained burn");
		fleet.getStats().getFleetwideMaxBurnMod().modifyMult(getModId(), burnMult, "Sustained burn");
		
		//fleet.getStats().getAccelerationMult().modifyMult(getModId(), ACCELERATION_MULT);
		fleet.getStats().getAccelerationMult().modifyMult(getModId(), 1f + (ACCELERATION_MULT - 1f) * level);
		
		for (FleetMemberViewAPI view : fleet.getViews()) {
			//view.getContrailColor().shift(getModId(), new Color(50,50,50,155), 1f, 1f, .5f);
			view.getContrailColor().shift(getModId(), view.getEngineColor().getBase(), 1f, 1f, 0.5f * level);
			view.getEngineGlowSizeMult().shift(getModId(), 1.5f, 1f, 1f, 1f * level);
			view.getEngineHeightMult().shift(getModId(), 3f, 1f, 1f, 1f * level);
			view.getEngineWidthMult().shift(getModId(), 2f, 1f, 1f, 1f * level);
		}
		

		if (level <= 0) {
			cleanupImpl();
		}
	}
	
	@Override
	protected void deactivateImpl() {
		//cleanupImpl();
	}
	
	@Override
	protected void cleanupImpl() {
		CampaignFleetAPI fleet = getFleet();
		if (fleet == null) return;
		
		fleet.getStats().getSensorRangeMod().unmodify(getModId());
		fleet.getStats().getDetectedRangeMod().unmodify(getModId());
		fleet.getStats().getFleetwideMaxBurnMod().unmodify(getModId());
		fleet.getStats().getAccelerationMult().unmodify(getModId());
	}
	
	@Override
	public float getActivationDays() {
		return 1f;
	}

	@Override
	public float getDeactivationDays() {
		return 0.5f;
	}

	@Override
	public boolean showProgressIndicator() {
		return false;
	}
	
	@Override
	public boolean showActiveIndicator() {
		return isActive();
	}

	
	@Override
	public float getActivateCooldownDays() {
		return 0f;
	}
	
	@Override
	public float getDeactivateCooldownDays() {
		return 0f;
	}
	
	@Override
	public void createTooltip(TooltipMakerAPI tooltip, boolean expanded) {
		Color gray = Misc.getGrayColor();
		Color highlight = Misc.getHighlightColor();
		
		String status = " (off)";
		if (turnedOn) {
			status = " (on)";
		}
		
		LabelAPI title = tooltip.addTitle("Go Dark" + status);
		title.highlightLast(status);
		title.setHighlightColor(gray);

		float pad = 10f;
		
		
//		tooltip.addPara("Turns off all non-essential systems, reducing the range" +
//				" at which the fleet can be detected by %s and reducing the maximum burn" +
//				" level by %s", pad, 
//				highlight,
//				"" + (int)((1f - DETECTABILITY_MULT) * 100f) + "%",
//				"" + (int)((1f - MAX_BURN_MULT) * 100f) + "%"
//		);
		tooltip.addPara("Disables the transponder when activated.", pad);
	}

	public boolean hasTooltip() {
		return true;
	}
	
	@Override
	public boolean isTooltipExpandable() {
		return false;
	}


}





