package com.fs.starfarer.api.impl.campaign.missions;

import org.apache.log4j.Logger;

import com.fs.starfarer.api.EveryFrameScript;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.MissionBoardAPI;
import com.fs.starfarer.api.campaign.RepLevel;
import com.fs.starfarer.api.campaign.econ.CommodityOnMarketAPI;
import com.fs.starfarer.api.campaign.econ.CommoditySpecAPI;
import com.fs.starfarer.api.campaign.econ.EconomyAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.characters.ImportantPeopleAPI;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.characters.ImportantPeopleAPI.PersonDataAPI;
import com.fs.starfarer.api.impl.campaign.CoreReputationPlugin.MissionCompletionRep;
import com.fs.starfarer.api.impl.campaign.events.CoreEventProbabilityManager;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.Ranks;
import com.fs.starfarer.api.util.IntervalUtil;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class MarketProcurementMissionCreator implements EveryFrameScript {
	public static Logger log = Global.getLogger(CoreEventProbabilityManager.class);
	
	
	private MissionBoardAPI board;
	private IntervalUtil tracker = new IntervalUtil(0.25f, .75f);
	
	public MarketProcurementMissionCreator() {
		board = Global.getSector().getMissionBoard();
	}
	
	
	public void advance(float amount) {
		float days = Global.getSector().getClock().convertToDays(amount);
		tracker.advance(days);
		if (tracker.intervalElapsed()) {
			int maxConcurrent = (int) Global.getSettings().getFloat("maxMarketProcurementConcurrent");
			int num = board.getNumMissions(MarketProcurementMission.class);
			if (num < maxConcurrent) {
				for (int i = 0; i < 3; i++) {
					if (createMission()) break;
				}
			}
		}
		
//		for (MissionAvailabilityAPI mission : board.getMissionsCopy()) {
//			System.out.println("Misson: " + mission.getMission().getName());
//		}
	}
	
	
	/**
	 * 1) Target market should have some demand for commodity.
	 * 2) Commodity should not be available in sufficient quantity in target market's system.
	 * @return
	 */
	protected boolean createMission() {
		String commodityId = pickCommodity();
		if (commodityId == null) return false;
		
		MarketAPI market = pickMarket(commodityId);
		if (market == null) return false;
		
		boolean illegal = market.isIllegal(commodityId);
		
		WeightedRandomPicker<Float> quantityPicker = new WeightedRandomPicker<Float>();
		
		quantityPicker.add(10f);
		quantityPicker.add(50f);
		quantityPicker.add(100f);
		
		if (market.getSize() >= 4) {
			quantityPicker.add(200f);
		}
		
		if (market.getSize() >= 5) {
			quantityPicker.add(500f);
		}
		
		if (market.getSize() >= 6) {
			quantityPicker.add(1000f);
		}
		
		if (market.getSize() >= 7) {
			quantityPicker.add(2000f);
		}
		
		if (market.getSize() >= 8) {
			quantityPicker.add(10000f);
		}
		
		float quantity = quantityPicker.pick();
		if (illegal) {
			quantity *= 0.1f;
			if (quantity < 10) quantity = 10;
		}
		
		WeightedRandomPicker<Float> durationPicker = new WeightedRandomPicker<Float>();
		durationPicker.add(10f);
		durationPicker.add(20f);
		durationPicker.add(30f);
		durationPicker.add(60f);
		
		float duration = durationPicker.pick();
		
		float price = market.getDemandPrice(commodityId, quantity, false);
		if (illegal) {
			price *= 2f;
		} else {
			price *= 1f;
		}
		
		float maxQuantity = 10000f;
		float maxDuration = 60f;

		price *= 1f + 0.5f * (1f - duration / maxDuration);
		price *= 1f + 0.5f * (1f - quantity / maxQuantity);
		
		
		float pricePerUnit  = (int) (price / quantity);
		if (pricePerUnit < 10) pricePerUnit = 10;
		
		
		boolean withBonus = duration > 10f && (float) Math.random() > 0.5f;
		
		float bonusDuration = 0f;
		float bonusAmount = 0f;
		if (withBonus) {
			bonusDuration = duration * 0.5f;
			if (bonusDuration < 10f) bonusDuration = 10f;
			
			bonusAmount = (int) (price * (0.5f + 1f * (float) Math.random()));
			bonusAmount *= Math.max(maxDuration - bonusDuration, 0) / maxDuration;
			bonusAmount = (int) (bonusAmount / 1000) * 1000;
			if (bonusAmount < 1000) bonusAmount = 1000;
		}
		
		float repAmount = 0.01f * price / 100000f;
		if (repAmount < 0.01f) repAmount = 0.01f;
		if (repAmount > 0.05f) repAmount = 0.05f;
		
		
		MissionCompletionRep rep = new MissionCompletionRep(repAmount, RepLevel.WELCOMING, -repAmount, RepLevel.INHOSPITABLE);
		
		PersonAPI contact = pickContact(market, market.getCommodityData(commodityId));
		
		MarketProcurementMission mission = new MarketProcurementMission(market.getId(), contact, 
												commodityId, pricePerUnit, quantity, bonusAmount,
												10f, duration, bonusDuration,
												rep);
		
		log.info("Created MarketProcurementMission: " + commodityId + " to " + market.getName());
		
		if (market.getContainingLocation() == null || market.getContainingLocation().isHyperspace()) {
			board.makeAvailableAt(mission, market.getId());
		} else {
			for (MarketAPI curr : Misc.getMarketsInLocation(market.getContainingLocation())) {
				//if (!curr.getFaction().isHostileTo(market.getFaction())) {
					board.makeAvailableAt(mission, curr.getId());
				//}
			}
		}
		
		float minRange = Global.getSettings().getFloat("missionPostingRangeMin");
		float maxRange = Global.getSettings().getFloat("missionPostingRangeMax");
		
		float range = minRange + (maxRange - minRange) * (float) Math.random();
		for (MarketAPI curr : Misc.getNearbyMarkets(market.getLocationInHyperspace(), range)) {
			if (!curr.getFaction().isHostileTo(market.getFaction())) {
				board.makeAvailableAt(mission, curr.getId());
			}
		}
		return true;
	}
	
	
	protected PersonAPI pickContact(MarketAPI market, CommodityOnMarketAPI com) {
		PersonAPI contact = null;
		ImportantPeopleAPI ip = Global.getSector().getImportantPeople();
		
		String comId = com.getId();
		
		if (com.getCommodity().hasTag(Commodities.TAG_MILITARY)) {
			if ((float) Math.random() > 0.1f) {
				contact = ip.getPerson(market.getFaction(), market,
							MarketProcurementMission.PERSON_CHECKOUT_REASON, Ranks.GROUND_LIEUTENANT, 
							Ranks.POST_SUPPLY_OFFICER,
							Ranks.POST_BASE_COMMANDER,
							Ranks.POST_OUTPOST_COMMANDER,
							Ranks.POST_PORTMASTER).getPerson();
			} else {
				contact = getCriminal(market, MarketProcurementMission.PERSON_CHECKOUT_REASON, Factions.PIRATES).getPerson();
			}
		}
		
		if (contact == null && com.getCommodity().hasTag(Commodities.TAG_MEDICAL)) {
			if ((float) Math.random() > 0.25f) {
				contact = ip.getPerson((float) Math.random() > 0.5f ? Factions.INDEPENDENT : market.getFactionId(),
						market,
						MarketProcurementMission.PERSON_CHECKOUT_REASON, Ranks.CITIZEN, 
						Ranks.POST_MEDICAL_SUPPLIER).getPerson();
			} else {
				contact = getCriminal(market, MarketProcurementMission.PERSON_CHECKOUT_REASON, Factions.PIRATES).getPerson();
			}
		}
		
		if (contact == null && com.getCommodity().hasTag(Commodities.TAG_LUXURY)) {
			if ((float) Math.random() > 0.1f && !market.isIllegal(comId)) {
				contact = getLegitTrader(market, MarketProcurementMission.PERSON_CHECKOUT_REASON, market.getFactionId()).getPerson();
			} else {
				contact = getCriminal(market, MarketProcurementMission.PERSON_CHECKOUT_REASON, Factions.PIRATES).getPerson();
			}
		}
		
		if (contact == null) {
			if ((float) Math.random() > 0.05f && !market.isIllegal(comId)) {
				contact = getLegitTrader(market, MarketProcurementMission.PERSON_CHECKOUT_REASON, market.getFactionId()).getPerson();
			} else {
				contact = getCriminal(market, MarketProcurementMission.PERSON_CHECKOUT_REASON, Factions.PIRATES).getPerson();
			}
		}
		
		if (contact == null) {
			// shouldn't happen, but just in case
			contact = market.getFaction().createRandomPerson();
			contact.setPostId(Ranks.POST_CITIZEN);
			contact.setRankId(null);
			market.addPerson(contact);
			ip.addPerson(contact);
			ip.getData(contact).getLocation().setMarket(market);
		}
		
		return contact;
	}
	
	
	public static PersonDataAPI getLegitTrader(MarketAPI market, String checkoutReason, String factionId) {
		ImportantPeopleAPI ip = Global.getSector().getImportantPeople();
		return ip.getPerson(factionId,
				market,
				checkoutReason, Ranks.CITIZEN, 
				Ranks.POST_MERCHANT,
				Ranks.POST_COMMODITIES_AGENT,
				Ranks.POST_INVESTOR,
				Ranks.POST_TRADER);
	}
	
	public static PersonDataAPI getCriminal(MarketAPI market, String checkoutReason, String factionId) {
		ImportantPeopleAPI ip = Global.getSector().getImportantPeople();
		return ip.getPerson(factionId,
				market,
				checkoutReason, Ranks.CITIZEN, 
				Ranks.POST_GANGSTER,
				Ranks.POST_SMUGGLER,
				Ranks.POST_FENCE);
	}
	
	
	
	
	protected String pickCommodity() {
		EconomyAPI economy = Global.getSector().getEconomy();
		WeightedRandomPicker<String> picker = new WeightedRandomPicker<String>();
		
		for (String curr : economy.getAllCommodityIds()) {
			CommoditySpecAPI spec = economy.getCommoditySpec(curr);
			if (spec.hasTag(Commodities.TAG_CREW)) continue;
			if (spec.hasTag(Commodities.TAG_MARINES)) continue;
//			if (spec.getId().equals(Commodities.SUPPLIES)) continue;
//			if (spec.getId().equals(Commodities.FUEL)) continue;
			
			float weight = spec.getBasePrice();
			picker.add(curr, weight);
		}
		
		return picker.pick();
	}
	
	protected MarketAPI pickMarket(String commodityId) {
		EconomyAPI economy = Global.getSector().getEconomy();
		
		WeightedRandomPicker<MarketAPI> picker = new WeightedRandomPicker<MarketAPI>();
		
		float minQty = 10f;
		if (commodityId.equals(Commodities.SUPPLIES)) {
			minQty = Math.max(minQty, Global.getSettings().getFloat("blackMarketMinSupplies") * 2f);
		}
		if (commodityId.equals(Commodities.FUEL)) {
			minQty = Math.max(minQty, Global.getSettings().getFloat("blackMarketMinFuel") * 2f);
		}
		
		for (MarketAPI market : economy.getMarketsCopy()) {
			CommodityOnMarketAPI com = market.getCommodityData(commodityId);
			
			if (com.getAverageStockpileAfterDemand() >= minQty) continue;
			if (com.getDemand().getDemandValue() < minQty) continue;
			
			if (doNearbyMarketsHave(market, commodityId, minQty)) continue;
			
			float weight = market.getSize();
			picker.add(market, weight);
		}
		
		return picker.pick();
	}
	
	protected boolean doNearbyMarketsHave(MarketAPI from, String commodityId, float minQty) {
		if (from.getContainingLocation() == null || from.getContainingLocation().isHyperspace()) return false;
		
		for (MarketAPI curr : Misc.getMarketsInLocation(from.getContainingLocation())) {
			if (curr == from) continue;
			if (curr.getPrimaryEntity() != null && from.getPrimaryEntity() != null) {
				float dist = Misc.getDistance(curr.getPrimaryEntity().getLocation(), from.getPrimaryEntity().getLocation());
				if (dist > 6000) continue;
			}
			
			CommodityOnMarketAPI com = curr.getCommodityData(commodityId);
			if (com.getAverageStockpileAfterDemand() >= minQty) return true;
			if (com.getSupplyValue() >= minQty) return true;
		}
		return false;
	}

	public boolean isDone() {
		return false;
	}

	public boolean runWhilePaused() {
		return false;
	}

}
