package com.fs.starfarer.api.impl.campaign.submarkets;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.CargoStackAPI;
import com.fs.starfarer.api.campaign.CoreUIAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.FleetDataAPI;
import com.fs.starfarer.api.campaign.PlayerMarketTransaction;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.SubmarketPlugin;
import com.fs.starfarer.api.campaign.CampaignUIAPI.CoreUITradeMode;
import com.fs.starfarer.api.campaign.econ.CommodityOnMarketAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.econ.SubmarketAPI;
import com.fs.starfarer.api.combat.WeaponAPI.AIHints;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.fleet.ShipRolePick;
import com.fs.starfarer.api.impl.campaign.ids.Conditions;
import com.fs.starfarer.api.impl.campaign.ids.ShipRoles;
import com.fs.starfarer.api.loading.WeaponSlotAPI;
import com.fs.starfarer.api.loading.WeaponSpecAPI;
import com.fs.starfarer.api.util.Highlights;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class BaseSubmarketPlugin implements SubmarketPlugin {

	public static class ShipSalesData {
		private String variantId;
		private float numShips;
		private float totalValue;
		public String getVariantId() {
			return variantId;
		}
		public void setVariantId(String variantId) {
			this.variantId = variantId;
		}
		public float getNumShips() {
			return numShips;
		}
		public void setNumShips(float numShips) {
			this.numShips = numShips;
		}
		public float getTotalValue() {
			return totalValue;
		}
		public void setTotalValue(float totalValue) {
			this.totalValue = totalValue;
		}
	}
	
	protected MarketAPI market;
	protected SubmarketAPI submarket;
	
	private CargoAPI cargo;
	protected float minCargoUpdateInterval = 30; // campaign days
	protected float sinceLastCargoUpdate = minCargoUpdateInterval + 1;
	
	protected Random itemGenRandom = new Random();
	
	public void init(SubmarketAPI submarket) {
		this.submarket = submarket;
		this.market = submarket.getMarket();
	}

	public String getName() {
		return null;
	}

	public CargoAPI getCargo() {
		if (cargo == null) {
			this.cargo = Global.getFactory().createCargo(true);
			this.cargo.initMothballedShips(submarket.getFaction().getId());
		}
		return cargo;
	}
	

	public void updateCargoPrePlayerInteraction() {
		
	}
	
	public void advance(float amount) {
		float days = Global.getSector().getClock().convertToDays(amount);
		sinceLastCargoUpdate += days;
		
	}
	
	public boolean okToUpdateCargo() {
		return sinceLastCargoUpdate >= minCargoUpdateInterval;
	}
	
	
	
	
	/**
	 * Complete and utter hack so that weapons delivered by convoys to show up
	 * in the first submarket (usually "open market").
	 * Not used anymore.
	 * 
	 */
	public void mirrorEntityWeaponsInSubmarketCargoHack() {
		for (SectorEntityToken entity : market.getConnectedEntities()) {
			if (entity.getCargo() == null) continue;
			
			for (CargoStackAPI stack : entity.getCargo().getStacksCopy()) {
				if (stack.isNull()) continue;
				if (stack.isBlueprintStack() || stack.isWeaponStack()) {
					getCargo().removeItems(stack.getType(), stack.getData(), stack.getSize());
				}
			}
			for (CargoStackAPI stack : entity.getCargo().getStacksCopy()) {
				if (stack.isNull()) continue;
				if (stack.isBlueprintStack() || stack.isWeaponStack()) {
					getCargo().addItems(stack.getType(), stack.getData(), stack.getSize());
				}
			}
		}
	}
	
	
	public void addAllCargo(CargoAPI otherCargo) {
		for (CargoStackAPI stack : otherCargo.getStacksCopy()) {
			if (stack.isNull()) continue;
			getCargo().addItems(stack.getType(), stack.getData(), stack.getSize());
		}
	}
	
	
	/**
	 * At least com.getAverageStockpileAfterDemand().
	 * @param com
	 * @return
	 */
	public float computeStockpileAvailableForSale(CommodityOnMarketAPI com) {
//		if (com.getId().equals("organs")) {
//			System.out.println("sfsdfsdfdsf");
//		}
		
		float available = com.getAverageStockpileAfterDemand();
		
		if (true) return available;
		
		available = Math.min(available, com.getSupplyValue());
		
		float stockpileUtility = com.getDemand().getRollingAverageStockpileUtility();
		float demand = com.getDemand().getDemandValue();
		
		float excessFraction = 0f;
		if (stockpileUtility > demand && stockpileUtility > 0 && demand > 0) {
			excessFraction = Math.max(0f, 1f - demand / stockpileUtility);
		}
		available = Math.max(available, com.getAverageStockpileAfterDemand() * excessFraction);

		if (!com.isPersonnel()) {
			float nonConsumingDemand = com.getDemand().getNonConsumingDemandValue();
			float nonConsumingFraction = 0f;
			if (nonConsumingDemand > 0 && demand > 0) {
				nonConsumingFraction = nonConsumingDemand / (nonConsumingDemand + demand);
				available = Math.max(available, com.getAverageStockpileAfterDemand() * nonConsumingFraction);
			}
		}
		
		return available;
	}

	public float getTariff() {
		return market.getTariff().getModifiedValue();
	}

	public String getBuyVerb() {
		return "Buy";
	}

	public String getSellVerb() {
		return "Sell";
	}

	public boolean isFreeTransfer() {
		return false;
	}

	public boolean isEnabled(CoreUIAPI ui) {
		return ui.getTradeMode() == CoreUITradeMode.OPEN || isBlackMarket();
		//return true;
	}
	public OnClickAction getOnClickAction(CoreUIAPI ui) {
		return OnClickAction.OPEN_SUBMARKET;
	}
	public String getDialogText(CoreUIAPI ui) {
		return null;
	}
	public Highlights getDialogTextHighlights(CoreUIAPI ui) {
		return null;
	}
	public DialogOption [] getDialogOptions(CoreUIAPI ui) {
		return null;
	}
	public String getTooltipAppendix(CoreUIAPI ui) {
		return null;
	}
	public Highlights getTooltipAppendixHighlights(CoreUIAPI ui) {
		return null;
	}
	

	public void reportPlayerMarketTransaction(PlayerMarketTransaction transaction) {
		if (!isParticipatesInEconomy()) return;
		
		// need to modify quantities by the rolling average factor
		// in order for price changes due to stockpile changes to remain consistent
		// with the internals of the economy.
		float factor = Global.getSettings().getFloat("economyRollingAverageFactor");
		
		for (CargoStackAPI stack : transaction.getSold().getStacksCopy()) {
			if (stack.isCommodityStack()) {
				float qty = stack.getSize() * factor;
				CommodityOnMarketAPI com = market.getCommodityData(stack.getCommodityId());
				com.addToAverageStockpile(qty * com.getPlayerTradeStockpileImpactMult());
				com.addToStockpile(qty * com.getPlayerTradeStockpileImpactMult());
			}
		}
		for (CargoStackAPI stack : transaction.getBought().getStacksCopy()) {
			if (stack.isCommodityStack()) {
				float qty = stack.getSize() * factor;
				CommodityOnMarketAPI com = market.getCommodityData(stack.getCommodityId());
				com.removeFromAverageStockpile(qty * com.getPlayerTradeStockpileImpactMult());
				com.removeFromStockpile(qty * com.getPlayerTradeStockpileImpactMult());
				//System.out.println("New average: " + com.getAverageStockpile());
			}
		}
	}

	public boolean isBlackMarket() {
		//return false;
		return market.getFaction().isHostileTo(submarket.getFaction());
	}

	public boolean isOpenMarket() {
		return false;
	}
	
	public boolean isParticipatesInEconomy() {
		return true;
	}
	

	public boolean isIllegalOnSubmarket(String commodityId, TransferAction action) {
		if (market.hasCondition(Conditions.FREE_PORT)) return false;
		//return market.isIllegal(commodityId); 
		return submarket.getFaction().isIllegal(commodityId); 
	}

	public boolean isIllegalOnSubmarket(CargoStackAPI stack, TransferAction action) {
		if (!stack.isCommodityStack()) return false;
		return isIllegalOnSubmarket((String) stack.getData(), action);
	}
	
	public String getIllegalTransferText(CargoStackAPI stack, TransferAction action) {
		return "Illegal to trade on the " + submarket.getNameOneLine().toLowerCase() + " here";
	}
	
	public boolean isIllegalOnSubmarket(FleetMemberAPI member, TransferAction action) {
		return false;
	}
	
	public String getIllegalTransferText(FleetMemberAPI member, TransferAction action) {
		//return "Illegal to trade on the " + submarket.getNameOneLine().toLowerCase() + " here";
		if (action == TransferAction.PLAYER_BUY) {
			return "Illegal to buy"; // this shouldn't happen
		} else {
			return "Illegal to sell";
		}
	}
	
	
	protected void addWeaponsBasedOnMarketSize(int baseCount, int maxTier, WeightedRandomPicker<FactionAPI> factionPicker) {
		int marketSize = market.getSize();
		addWeaponsForRolePicks(baseCount, maxTier, factionPicker,
				ShipRoles.COMBAT_SMALL, 
				ShipRoles.COMBAT_SMALL, 
				ShipRoles.COMBAT_SMALL, 
				ShipRoles.ESCORT_SMALL, 
				ShipRoles.FREIGHTER_SMALL, 
				ShipRoles.COMBAT_FREIGHTER_SMALL, 
				ShipRoles.CARRIER_SMALL);
		if (marketSize >= 2) {
			addWeaponsForRolePicks(baseCount, maxTier, factionPicker,
					ShipRoles.COMBAT_MEDIUM,
					ShipRoles.COMBAT_MEDIUM, 
					ShipRoles.ESCORT_MEDIUM, 
					ShipRoles.ESCORT_MEDIUM, 
					ShipRoles.FREIGHTER_MEDIUM, 
					ShipRoles.COMBAT_FREIGHTER_MEDIUM, 
					ShipRoles.CARRIER_MEDIUM);
		}
		if (marketSize >= 4) {
			addWeaponsForRolePicks(baseCount, maxTier, factionPicker,
					ShipRoles.CARRIER_LARGE,
					ShipRoles.FREIGHTER_LARGE,
					ShipRoles.COMBAT_FREIGHTER_LARGE,
					ShipRoles.COMBAT_LARGE,
					ShipRoles.COMBAT_LARGE);
		}
		if (marketSize >= 6) {
			addWeaponsForRolePicks(baseCount, maxTier, factionPicker,
					ShipRoles.CARRIER_LARGE,
					ShipRoles.COMBAT_CAPITAL,
					ShipRoles.COMBAT_CAPITAL);
		}

	}
	
	protected void addWeaponsForRolePicks(int maxCount, int maxTier, WeightedRandomPicker<FactionAPI> factionPicker, String ... roles) {
		WeightedRandomPicker<String> picker = new WeightedRandomPicker<String>();
		if (!Global.getSettings().isDevMode() || true) {
			picker.setRandom(itemGenRandom);
		}
		
		for (String role : roles) {
			List<WeaponSpecAPI> weapons = getWeaponsOnRolePick(role, factionPicker);
			for (WeaponSpecAPI spec : weapons) {
				if (spec.getTier() > maxTier) continue;
				picker.add(spec.getWeaponId(), spec.getSize().ordinal() + 1);
			}
		}
		
		CargoAPI cargo = getCargo();
		int picks = 0;
		while (!picker.isEmpty() && picks < maxCount) {
			String id = picker.pick();
			cargo.addWeapons(id, 2);
			picker.remove(id);
			picks++;
		}
	}
	
	protected List<WeaponSpecAPI> getWeaponsOnRolePick(String role, WeightedRandomPicker<FactionAPI> factionPicker) {
		float qf = market.getShipQualityFactor();
		
		List<WeaponSpecAPI> result = new ArrayList<WeaponSpecAPI>();
		
		FactionAPI faction = submarket.getFaction();
		if (factionPicker != null && !factionPicker.isEmpty()) {
			faction = factionPicker.pick();
		}
		List<ShipRolePick> picks = submarket.getFaction().pickShip(role, qf, itemGenRandom);
		for (ShipRolePick pick : picks) {
			FleetMemberType type = FleetMemberType.SHIP;
			if (pick.isFighterWing()) {
				//type = FleetMemberType.FIGHTER_WING;
				continue;
			}
			
			FleetMemberAPI member = Global.getFactory().createFleetMember(type, pick.variantId);
			
			for (String slotId : member.getVariant().getFittedWeaponSlots()) {
				WeaponSlotAPI slot = member.getVariant().getSlot(slotId);
				if (slot.isBuiltIn() || slot.isSystemSlot() || slot.isDecorative()) continue;
				WeaponSpecAPI spec = member.getVariant().getWeaponSpec(slotId);
				if (spec.getAIHints().contains(AIHints.SYSTEM)) continue;
				result.add(spec);
			}
		}
		return result;
	}
	
	protected void addRandomWeapons(int max, int maxTier) {
		CargoAPI cargo = getCargo();
		List<String> weaponIds = Global.getSector().getAllWeaponIds();
		
		WeightedRandomPicker<String> picker = new WeightedRandomPicker<String>();
		if (!Global.getSettings().isDevMode() || true) {
			picker.setRandom(itemGenRandom);
		}
		
		float qualityFactor = market.getShipQualityFactor(); 
		for (String id : weaponIds) {
			WeaponSpecAPI spec = Global.getSettings().getWeaponSpec(id);
			if (spec.getTier() <= maxTier) {
				float weaponQuality = 0.33f * (float) spec.getTier();
				float qualityDiff = Math.abs(qualityFactor - weaponQuality);
				float f = Math.max(0, 1f - qualityDiff);
				float weight = Math.max(0.05f, f * f);
				picker.add(spec.getWeaponId(), weight);
			}
		}
		if (!picker.isEmpty()) {
			for (int i = 0; i < max; i++) {
				String weaponId = picker.pick();
				int quantity = 2;
				cargo.addWeapons(weaponId, quantity);
			}	
		}
		
//		for (int i = 0; i < max; i++) {
//			String weaponId = (String) weaponIds.get((int) (weaponIds.size() * Math.random()));
//			//int quantity = (int)(Math.random() * 4f + 2f);
//			int quantity = 1;
//			cargo.addWeapons(weaponId, quantity);
//		}
	}
	
	protected void pruneWeapons(float mult) {
		CargoAPI cargo = getCargo();
		for (CargoStackAPI stack : cargo.getStacksCopy()) {
			if (!stack.isWeaponStack()) continue;
			
			float qty = stack.getSize();
			if (qty <= 1) {
				if (itemGenRandom.nextFloat() > mult) {
					cargo.removeItems(stack.getType(), stack.getData(), 1);
				}
			} else {
				cargo.removeItems(stack.getType(), stack.getData(), Math.round(qty * mult));
			}
		}
	}
	
	protected void addShip(String variantOrWingId) {
		FleetMemberAPI member = null;
		if (variantOrWingId.endsWith("_wing")) {
			member = Global.getFactory().createFleetMember(FleetMemberType.FIGHTER_WING, variantOrWingId);
		} else {
			member = Global.getFactory().createFleetMember(FleetMemberType.SHIP, variantOrWingId);
		}
		member.getRepairTracker().setMothballed(true);
		getCargo().getMothballedShips().addFleetMember(member);
	}
	

	protected void addShipsForRoles(int maxTotal, WeightedRandomPicker<String> rolePicker, WeightedRandomPicker<FactionAPI> factionPicker) {
		if (rolePicker.isEmpty()) return;
		
		//if (!Global.getSettings().isDevMode()) {
			rolePicker.setRandom(itemGenRandom);
			if (factionPicker != null) {
				factionPicker.setRandom(itemGenRandom);
			}
		//}
		
		int curr = getCargo().getMothballedShips().getMembersListCopy().size();
		int toAdd = maxTotal - curr;
		int added = 0;
		while (added < toAdd) {
			String role = rolePicker.pick();
			addShipsForRole(role, rolePicker.getRandom(), factionPicker);
			added++;
		}
	}
	
	protected void addShipsForRole(String role, Random random, WeightedRandomPicker<FactionAPI> factionPicker) {
		float qf = market.getShipQualityFactor();
		FactionAPI faction = submarket.getFaction();
		if (factionPicker != null && !factionPicker.isEmpty()) {
			faction = factionPicker.pick();
		}
		List<ShipRolePick> picks = faction.pickShip(role, qf, random);
		for (ShipRolePick pick : picks) {
			FleetMemberType type = FleetMemberType.SHIP;
			String variantId = pick.variantId;
			if (pick.isFighterWing()) {
				type = FleetMemberType.FIGHTER_WING;
			} else {
				FleetMemberAPI member = Global.getFactory().createFleetMember(type, pick.variantId);
				variantId = member.getHullId() + "_Hull";
			}
			FleetMemberAPI member = Global.getFactory().createFleetMember(type, variantId);
			member.getRepairTracker().setMothballed(true);
			getCargo().getMothballedShips().addFleetMember(member);
		}
	}

	
	protected void pruneShips(float mult) {
		CargoAPI cargo = getCargo();
		FleetDataAPI data = cargo.getMothballedShips();
		for (FleetMemberAPI member : data.getMembersListCopy()) {
			if (itemGenRandom.nextFloat() > mult) {
				data.removeFleetMember(member);
			}
		}
	}
}







