package com.fs.starfarer.api.impl.campaign.terrain;

import java.awt.Color;

import org.lwjgl.opengl.GL11;
import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.graphics.SpriteAPI;
import com.fs.starfarer.api.util.Misc;

public class AuroraRenderer {
	public static interface AuroraRendererDelegate {
		float getAuroraInnerRadius();
		float getAuroraOuterRadius();
		Vector2f getAuroraCenterLoc();
		
		Color getAuroraColorForAngle(float angle);
		float getAuroraAlphaMultForAngle(float angle);
		
		float getAuroraShortenMult(float angle);
		float getAuroraInnerOffsetMult(float angle);
		
		float getAuroraThicknessMult(float angle);
		float getAuroraThicknessFlat(float angle);
		
		
		float getAuroraTexPerSegmentMult();
		float getAuroraBandWidthInTexture();
		
		SpriteAPI getAuroraTexture();
	}
	
	private AuroraRendererDelegate delegate;
	private float phaseAngle;
	public AuroraRenderer(AuroraRendererDelegate delegate) {
		this.delegate = delegate;
	}

	
	public void advance(float amount) {
		float days = Global.getSector().getClock().convertToDays(amount);
		phaseAngle += days * 360f * 0.5f;
		phaseAngle = Misc.normalizeAngle(phaseAngle);
	}
	
	
	public void render(float alphaMult) {
		float bandWidthInTexture = delegate.getAuroraBandWidthInTexture();
		float bandIndex;
		
		float radStart = delegate.getAuroraInnerRadius();
		float radEnd = delegate.getAuroraOuterRadius();;
		
		if (radEnd < radStart + 10f) radEnd = radStart + 10f;
		
		float circ = (float) (Math.PI * 2f * (radStart + radEnd) / 2f);
		float pixelsPerSegment = 50f;
		float segments = Math.round(circ / pixelsPerSegment);
		
		float startRad = (float) Math.toRadians(0);
		float endRad = (float) Math.toRadians(360f);
		float spanRad = Math.abs(endRad - startRad);
		float anglePerSegment = spanRad / segments;
		
		Vector2f loc = delegate.getAuroraCenterLoc();
		float x = loc.x;
		float y = loc.y;

		
		GL11.glPushMatrix();
		GL11.glTranslatef(x, y, 0);
		
		GL11.glEnable(GL11.GL_TEXTURE_2D);
		
		delegate.getAuroraTexture().bindTexture();
		
		GL11.glEnable(GL11.GL_BLEND);
		GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE);
		
		float thickness = (radEnd - radStart) * 1f;
		float radius = radStart;

		float texProgress = 0f;
		float texHeight = delegate.getAuroraTexture().getTextureHeight();
		float imageHeight = delegate.getAuroraTexture().getHeight();
		float texPerSegment = pixelsPerSegment * texHeight / imageHeight * bandWidthInTexture / thickness;
		
		texPerSegment *= delegate.getAuroraTexPerSegmentMult();
		
		float totalTex = Math.max(1f, Math.round(texPerSegment * segments));
		texPerSegment = totalTex / segments;
		
		float texWidth = delegate.getAuroraTexture().getTextureWidth();
		float imageWidth = delegate.getAuroraTexture().getWidth();
		
//		GL11.glDisable(GL11.GL_TEXTURE_2D);
//		GL11.glDisable(GL11.GL_BLEND);
//		GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_LINE);
//		GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_FILL);
		
//		CampaignFleetAPI playerFleet = Global.getSector().getPlayerFleet();
//		SectorEntityToken star = ((StarSystemAPI)playerFleet.getContainingLocation()).getStar();
//		float distToStar = Misc.getDistance(playerFleet.getLocation(), star.getLocation());// - star.getRadius();
//		System.out.println("Dist to outer star: " + distToStar);
//		System.out.println("Aurora outer: " + delegate.getAuroraOuterRadius());
		
		for (int iter = 0; iter < 2; iter++) {
			if (iter == 0) {
				bandIndex = 1;
			} else {
				bandIndex = 0;
			}
			
			float leftTX = (float) bandIndex * texWidth * bandWidthInTexture / imageWidth;
			float rightTX = (float) (bandIndex + 1f) * texWidth * bandWidthInTexture / imageWidth - 0.001f;
			
			GL11.glBegin(GL11.GL_QUAD_STRIP);
			for (float i = 0; i < segments + 1; i++) {
				
				float segIndex = i % (int) segments;
				
				//float phaseAngleRad = (float) Math.toRadians(phaseAngle + segIndex * 10) + (segIndex * anglePerSegment * 10f);
				float phaseAngleRad;
				if (iter == 0) {
					phaseAngleRad = (float) Math.toRadians(phaseAngle) + (segIndex * anglePerSegment * 10f);
				} else { //if (iter == 1) { 
					phaseAngleRad = (float) Math.toRadians(-phaseAngle) + (segIndex * anglePerSegment * 5f);
				}
				
				
				float angle = (float) Math.toDegrees(segIndex * anglePerSegment);
				if (iter == 1) angle += 180;
				
				float pulseSin = (float) Math.sin(phaseAngleRad);
				float pulseAmount = pulseSin * thickness * delegate.getAuroraShortenMult(angle);
				float pulseInner = pulseAmount * 0.1f;
				pulseInner *= delegate.getAuroraInnerOffsetMult(angle);
				//pulseInner *= Math.max(0, pulseSin - 0.5f);
				//pulseInner *= 0f;
				
				float r = radius;

				float thicknessMult = delegate.getAuroraThicknessMult(angle);
				float thicknessFlat = delegate.getAuroraThicknessFlat(angle);
				
				float theta = anglePerSegment * segIndex;;
				float cos = (float) Math.cos(theta);
				float sin = (float) Math.sin(theta);
				float x1 = cos * (r - pulseInner);
				float y1 = sin * (r - pulseInner);
				float x2 = cos * (r + thickness * thicknessMult - pulseAmount + thicknessFlat);
				float y2 = sin * (r + thickness * thicknessMult - pulseAmount + thicknessFlat);
				
				x2 += (float) (Math.cos(phaseAngleRad) * pixelsPerSegment * 0.33f);
				y2 += (float) (Math.sin(phaseAngleRad) * pixelsPerSegment * 0.33f);
				
				Color color = delegate.getAuroraColorForAngle(angle);
				float alpha = delegate.getAuroraAlphaMultForAngle(angle);
				GL11.glColor4ub((byte)color.getRed(),
						(byte)color.getGreen(),
						(byte)color.getBlue(),
						(byte)((float) color.getAlpha() * alphaMult * alpha));
				
				GL11.glTexCoord2f(leftTX, texProgress);
				GL11.glVertex2f(x1, y1);
				GL11.glTexCoord2f(rightTX, texProgress);
				GL11.glVertex2f(x2, y2);
				
				texProgress += texPerSegment;
			}
			GL11.glEnd();
			
			GL11.glRotatef(180, 0, 0, 1);
		}
		GL11.glPopMatrix();
		
//		GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_FILL);
	}
	
}
