package com.fs.starfarer.api.impl.campaign.terrain;

import java.util.Random;

import com.fs.starfarer.api.util.IntervalUtil;

public class HyperspaceAutomaton {

	protected int [][] cells;
	protected int [][] next;

	protected IntervalUtil interval;
	protected boolean doneWithIteration = false;
	protected int currentColumn = 0;
	public HyperspaceAutomaton(int width, int height, float minInterval, float maxInterval) {
		cells = new int [width][height];
		next = new int [width][height];
		
		interval = new IntervalUtil(minInterval, maxInterval);
		
		setLive(0.1f);
		interval.forceIntervalElapsed();
	}
	
	
	public IntervalUtil getInterval() {
		return interval;
	}

	public void advance(float days) {
		updateCells(days);
		
		interval.advance(days);
		if (interval.intervalElapsed()) {
			//System.out.println("Max reached:" + currentColumn);
			doneWithIteration = false;
			//setLive(0.001f);
			setLive(0.01f);
			cells = next;
			next = new int [cells.length][cells[0].length];
			for (int i = 0; i < next.length; i++) {
				for (int j = 0; j < next[0].length; j++) {
					next[i][j] = cells[i][j];
				}
			}
			currentColumn = 0;
		}
	}
	
	
	public int[][] getCells() {
		return cells;
	}

	protected void updateCells(float days) {
		if (currentColumn >= cells.length) return;
		
		int chunk = (int) ((days * 1.5f) / interval.getMinInterval() * cells.length);
		if (chunk < 1) chunk = 1;
		int maxColumn = currentColumn + chunk;
		
		// 0: dead
		// 1: live
		// 2: dying
		for (int i = currentColumn; i < cells.length && i < maxColumn; i++) {
			for (int j = 0; j < cells[0].length; j++) {
				int val = (int) cells[i][j];
				if (val == 0) {
					int count = getLiveCountAround(i, j);
					if (count == 2) {
 						next[i][j] = 1;
					}
//					else if ((float) Math.random() < getRandomLifeChance()) {
//						next[i][j] = 1;
//					}
				} else if (val == 1) {
					next[i][j] = 2;
				} else if (val == 2) {
					next[i][j] = 0;
				}
				
//				int val = (int) cells[i][j];
//				int count = getLiveCountAround(i, j);
//				if (count < 2) {
//					next[i][j] = 0;
//				} else if (val != 0 && (count == 2 || count == 3)) {
//					// nothing happens
//				} else if (count > 3) { 
//					next[i][j] = 0;
//				} else if (val == 0 && count == 3) {
//					next[i][j] = 1;
//				}
//				
//				if ((float) Math.random() < getRandomLifeChance()) {
//					next[i][j] = 1;
//				}
			}
		}
		
		currentColumn = maxColumn;
	}
	
	public void setLive(float fraction) {
		float count = (float) (next.length * next[0].length) * fraction;
		if (count <= 0) {
			count = (float) Math.random() < count ? 1 : 0;
		}
		
		Random r = new Random();
		for (int i = 0; i < count; i++) {
			int x = r.nextInt(next.length);
			int y = r.nextInt(next[0].length);
			next[x][y] = 1;
		}
	}
	
//	protected float getRandomLifeChance() {
//		return 0.001f;
//	}
	
	protected int getLiveCountAround(int x, int y) {
		int count = 0;
		for (int i = Math.max(0, x - 1); i <= Math.min(x + 1, cells.length - 1); i++) {
			for (int j = Math.max(0, y - 1); j <= Math.min(y + 1, cells[0].length - 1); j++) {
				if (i == x && j == y) continue;
				if (cells[i][j] == 1) {
					count++;
				}
			}
		}
		return count;
	}
}






