package com.fs.starfarer.api.util;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import javax.imageio.ImageIO;

import org.lwjgl.opengl.GL11;
import org.lwjgl.util.vector.Vector2f;
import org.lwjgl.util.vector.Vector3f;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CampaignClockAPI;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.InteractionDialogAPI;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.events.CampaignEventManagerAPI;
import com.fs.starfarer.api.campaign.events.CampaignEventPlugin;
import com.fs.starfarer.api.campaign.events.CampaignEventTarget;
import com.fs.starfarer.api.campaign.rules.MemKeys;
import com.fs.starfarer.api.campaign.rules.MemoryAPI;
import com.fs.starfarer.api.combat.CombatEntityAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.impl.campaign.RuleBasedInteractionDialogPluginImpl;
import com.fs.starfarer.api.impl.campaign.events.BaseEventPlugin.MarketFilter;
import com.fs.starfarer.api.impl.campaign.ids.Difficulties;
import com.fs.starfarer.api.impl.campaign.ids.HullMods;
import com.fs.starfarer.api.impl.campaign.ids.MemFlags;
import com.fs.starfarer.api.impl.campaign.ids.Strings;
import com.fs.starfarer.api.impl.campaign.ids.Terrain;
import com.fs.starfarer.api.impl.campaign.rulecmd.unsetAll;
import com.fs.starfarer.api.impl.campaign.terrain.AsteroidSource;
import com.fs.starfarer.api.impl.campaign.terrain.NebulaTerrainPlugin;
import com.fs.starfarer.api.impl.campaign.terrain.BaseTiledTerrain.TileParams;


public class Misc {

	public static enum TokenType {
		VARIABLE,
		LITERAL,
		OPERATOR,
	}
	
	public static final Vector2f ZERO = new Vector2f(0, 0);
	
	public static class VarAndMemory {
		public String name;
		public MemoryAPI memory;
	}
	public static class Token {
		public String string;
		public TokenType type;
		public String varNameWithoutMemoryKeyIfKeyIsValid = null;
		public String varMemoryKey = null;
		public Token(String string, TokenType type) {
			this.string = string;
			this.type = type;
			
			if (isVariable()) {
				int index = string.indexOf(".");
				if (index > 0 && index < string.length() - 1) {
					varMemoryKey = string.substring(1, index);
					varNameWithoutMemoryKeyIfKeyIsValid = "$" + string.substring(index + 1);
				}
			}
		}

		public VarAndMemory getVarNameAndMemory(Map<String, MemoryAPI> memoryMap) {
			String varName = varNameWithoutMemoryKeyIfKeyIsValid;
			MemoryAPI memory = memoryMap.get(varMemoryKey);
			if (memory == null) {
				varName = string;
				memory = memoryMap.get(MemKeys.LOCAL);
			}
			if (memory == null) {
				throw new RuleException("No memory found for keys: " + varMemoryKey + ", " + MemKeys.LOCAL);
			}
			
			VarAndMemory result = new VarAndMemory();
			result.name = varName;
			result.memory = memory;
			return result;
		}
		
		public String getStringWithTokenReplacement(String ruleId, InteractionDialogAPI dialog, Map<String, MemoryAPI> memoryMap) {
			String text = getString(memoryMap);
			if (text == null) return null;
			text = Global.getSector().getRules().performTokenReplacement(ruleId, text, dialog.getInteractionTarget(), memoryMap);
//			Map<String, String> tokens = Global.getSector().getRules().getTokenReplacements(ruleId, dialog.getInteractionTarget(), memoryMap);
//			for (String token : tokens.keySet()) {
//				String value = tokens.get(token);
//				text = text.replaceAll("(?s)\\" + token, value);
//			}
//			text = Misc.replaceTokensFromMemory(text, memoryMap);
			return text;
		}
		public String getString(Map<String, MemoryAPI> memoryMap) {
			String string = null;
			if (isVariable()) {
				VarAndMemory var = getVarNameAndMemory(memoryMap);
				string = var.memory.getString(var.name);
			} else {
				string = this.string;
			}
			return string;
		}
		
		public boolean getBoolean(Map<String, MemoryAPI> memoryMap) {
			String str = getString(memoryMap);
			return Boolean.parseBoolean(str);
		}
		
		public float getFloat(Map<String, MemoryAPI> memoryMap) {
			float result = 0f;
			if (isVariable()) {
				VarAndMemory var = getVarNameAndMemory(memoryMap);
				result = var.memory.getFloat(var.name);
			} else {
				result = Float.parseFloat(string);
			}
			return result;
		}
		
		public Color getColor(Map<String, MemoryAPI> memoryMap) {
			String string = getString(memoryMap);
			try {
				String [] parts = string.split(Pattern.quote(","));
				return new Color(Integer.parseInt(parts[0]),
						Integer.parseInt(parts[1]),
						Integer.parseInt(parts[2]),
						Integer.parseInt(parts[3]));
			} catch (Exception e) {
				return Global.getSettings().getColor(string);
			}
		}
		
		public boolean isLiteral() {
			return type == TokenType.LITERAL;
		}
		public boolean isVariable() {
			return type == TokenType.VARIABLE;
		}
		public boolean isOperator() {
			return type == TokenType.OPERATOR;
		}
		@Override
		public String toString() {
			if (isVariable()) {
				return string + " (" + type.name() + ", memkey: " + varMemoryKey + ", name: " + varNameWithoutMemoryKeyIfKeyIsValid + ")";
			} else {
				return string + " (" + type.name() + ")";
			}
		}
	}
	
	public static List<Token> tokenize(String string) {
		List<Token> result = new ArrayList<Token>();
		boolean inQuote = false;
		boolean inOperator = false;
		
		StringBuffer currToken = new StringBuffer();
		for (int i = 0; i < string.length(); i++) {
			char curr = string.charAt(i);
			char next = 0;
			if (i + 1 < string.length()) next = string.charAt(i + 1);
			boolean charEscaped = false;
			if (curr == '\\') {
				i++;
				if (i >= string.length()) {
					throw new RuleException("Escape character at end of string in: [" + string + "]");
				}
				curr = string.charAt(i);
				if (i + 1 < string.length()) next = string.charAt(i + 1);
				charEscaped = true;
			}
//			if (charEscaped) {
//				System.out.println("dfsdfs");
//			}
			
			if (curr == '"' && !charEscaped) {
				inQuote = !inQuote;
				if (!inQuote && currToken.length() <= 0) {
					result.add(new Token("", TokenType.LITERAL));
				} else if (currToken.length() > 0) {
					String str = currToken.toString();
					if (!inQuote) {
						result.add(new Token(str, TokenType.LITERAL));
					} else {
						if (str.startsWith("$")) {
							result.add(new Token(str, TokenType.VARIABLE));
						} else if (inOperator) {
							result.add(new Token(str, TokenType.OPERATOR));
						} else {
							result.add(new Token(str, TokenType.LITERAL));
						}
					}
				}
				inOperator = false;
				currToken.delete(0, 1000000);
				continue;
			}
			
			if (!inQuote && (curr == ' ' || curr == '\t')) {
				if (currToken.length() > 0) {
					String str = currToken.toString();
					if (str.startsWith("$")) {
						result.add(new Token(str, TokenType.VARIABLE));
					} else if (inOperator) {
						result.add(new Token(str, TokenType.OPERATOR));
					} else {
						result.add(new Token(str, TokenType.LITERAL));
					}
				}
				inOperator = false;
				currToken.delete(0, 1000000);
				continue;
			}
			
			if (!inQuote && !inOperator && isOperatorChar(curr) && (curr != '-' || !isDigit(next))) {
				if (currToken.length() > 0) {
					String str = currToken.toString();
					if (str.startsWith("$")) {
						result.add(new Token(str, TokenType.VARIABLE));
					} else {
						result.add(new Token(str, TokenType.LITERAL));
					}
				}
				currToken.delete(0, 1000000);
				inOperator = true;
				if (charEscaped && curr == 'n') {
					currToken.append("\n");
				} else {
					currToken.append(curr);
				}
				continue;
			}
			
			if (!inQuote && inOperator && !isOperatorChar(curr)) {
				if (currToken.length() > 0) {
					String str = currToken.toString();
					result.add(new Token(str, TokenType.OPERATOR));
				}
				currToken.delete(0, 1000000);
				inOperator = false;
				if (charEscaped && curr == 'n') {
					currToken.append("\n");
				} else {
					currToken.append(curr);
				}
				continue;
			}
			
			if (charEscaped && curr == 'n') {
				currToken.append("\n");
			} else {
				currToken.append(curr);
			}
		}
		
		if (inQuote) {
			throw new RuleException("Unmatched quotes in string: " + string + "]");
		}
		
		if (currToken.length() > 0) {
			String str = currToken.toString();
			if (str.startsWith("$")) {
				result.add(new Token(str, TokenType.VARIABLE));
			} else if (inOperator) {
				result.add(new Token(str, TokenType.OPERATOR));
			} else {
				result.add(new Token(str, TokenType.LITERAL));
			}
		}
	
		return result;
	}
	
	
	private static boolean isDigit(char c) {
		if (c == 0) return false;
		String digits = "1234567890";
		return digits.contains("" + c);
	}
	private static boolean isOperatorChar(char c) {
		String operatorChars = "=<>!+-";
		return operatorChars.contains("" + c);
	}
	
	
	public static String ucFirst(String str) {
		if (str == null) return "Null";
		if (str.isEmpty()) return "";
		return ("" + str.charAt(0)).toUpperCase() + str.substring(1);
	}
	
	public static String lcFirst(String str) {
		if (str == null) return "Null";
		if (str.isEmpty()) return "";
		return ("" + str.charAt(0)).toLowerCase() + str.substring(1);
	}
	
	
	public static String replaceTokensFromMemory(String text, Map<String, MemoryAPI> memoryMap) {
		for (String key : memoryMap.keySet()) {
			MemoryAPI memory = memoryMap.get(key);
			List<String> keys = new ArrayList<String>(memory.getKeys());
			Collections.sort(keys, new Comparator<String>() {
				public int compare(String o1, String o2) {
					return o2.length() - o1.length();
				}
			});
			for (String token : keys) {
				Object value = memory.get(token);
				if (value == null) value = "null";
				if (value instanceof String || value instanceof Boolean || value instanceof Float || value instanceof Integer) {
					text = text.replaceAll("(?s)\\$" + Pattern.quote(key) + "\\." + Pattern.quote(token.substring(1)), value.toString());
					text = text.replaceAll("(?s)\\$" + Pattern.quote(token.substring(1)), value.toString());
				}
			}
		}
		return text;
	}
	
	
	
	private static Vector2f temp3 = new Vector2f();
	public static float getDistance(Vector2f v1, Vector2f v2) {
		return Vector2f.sub(v1, v2, temp3).length();
	}
	
	public static float getDistance(float x1, float y1, float x2, float y2) {
		float xDiff = Math.abs(x1 - x2);
		float yDiff = Math.abs(y1 - y2);
		return (float) Math.sqrt(xDiff * xDiff + yDiff * yDiff);
	}
	
	public static float getDistanceLY(Vector2f v1, Vector2f v2) {
		return Vector2f.sub(v1, v2, temp3).length() / getUnitsPerLightYear();
	}
	
	public static float getRounded(float in) {
		if (in <= 10) return Math.max(1, (int) in);
		float pow = (int) Math.log10(in);
		float div = (float) Math.pow(10, Math.max(0, pow - 1));
		if (pow == 1) div = 10;
		return (int) Math.round(in / div) * div;
	}
	
	public static String getRoundedValue(float value) {
		if (Math.abs((float)Math.round(value) - value) < 0.0001f) {
			//if (value > 10 || value < -10) {
				return String.format("%d", (int) Math.round(value));
			//} else {
				//return String.format("%.1f", value);
			//}
		} else if ((int) Math.round((value * 100f)) == (int) Math.round((value * 10f)) * 10) {
			return (value > 10 || value < -10) ? "" + (int) Math.round(value) : String.format("%.1f", value);
		} else {
			return (value > 10 || value < -10) ? "" + (int) Math.round(value) : String.format("%.2f", value);
		}
	}
	
	public static String getRoundedValueMaxOneAfterDecimal(float value) {
		if (Math.abs((float)Math.round(value) - value) < 0.0001f) {
			return String.format("%d", (int) Math.round(value));
		} else if ((int) Math.round((value * 100f)) == (int) Math.round((value * 10f)) * 10) {
			return (value >= 10 || value <= -10) ? "" + (int) Math.round(value) : String.format("%.1f", value);
		} else {
			return (value >= 10 || value <= -10) ? "" + (int) Math.round(value) : String.format("%.1f", value);
		}
	}
	
	

	public static float logOfBase(float base, float num) {
	    return (float) (Math.log(num) / Math.log(base));
	}

	public static Vector2f getPointAtRadius(Vector2f from, float r) {
		float angle = (float) ((float) Math.random() * Math.PI * 2f);
		float x = (float) (Math.cos(angle) * r) + from.x;
		float y = (float) (Math.sin(angle) * r) + from.y;
		return new Vector2f(x, y);
	}
	
	public static Vector2f getPointWithinRadius(Vector2f from, float r) {
		r = r * (float) Math.random();
		float angle = (float) ((float) Math.random() * Math.PI * 2f);
		float x = (float) (Math.cos(angle) * r) + from.x;
		float y = (float) (Math.sin(angle) * r) + from.y;
		return new Vector2f(x, y);
	}
	
	public static float getSnapshotFPLost(CampaignFleetAPI fleet) {
		float fp = fleet.getFleetPoints();
		float before = 0;
		for (FleetMemberAPI member : fleet.getFleetData().getSnapshot()) {
			before += member.getFleetPointCost();
		}
		
		return before - fp;
	}
	
	public static List<FleetMemberAPI> getSnapshotMembersLost(CampaignFleetAPI fleet) {
		List<FleetMemberAPI> lost = new ArrayList<FleetMemberAPI>();
		List<FleetMemberAPI> curr = fleet.getFleetData().getMembersListCopy();
		for (FleetMemberAPI member : fleet.getFleetData().getSnapshot()) {
			if (!curr.contains(member)) {
				lost.add(member);
			}
		}
		
		return lost;
	}
	
	
	public static CampaignEventPlugin startEvent(CampaignEventTarget eventTarget, String eventId, Object params) {
		CampaignEventManagerAPI manager = Global.getSector().getEventManager();
		CampaignEventPlugin event = manager.getOngoingEvent(eventTarget, eventId);
		if (event == null) {
			event = manager.startEvent(eventTarget, eventId, params);
		}
		return event;
	}
	
	public static Color getHighlightColor() {
		return Global.getSettings().getColor("buttonShortcut");
	}
	public static Color getNegativeHighlightColor() {
		return Global.getSettings().getColor("textEnemyColor");
	}
	
	public static Color getPositiveHighlightColor() {
		return Global.getSettings().getColor("textFriendColor");
	}
	
	public static Color getGrayColor() {
		return Global.getSettings().getColor("textGrayColor");
	}
	
	public static Color getTextColor() {
		return Global.getSettings().getColor("standardTextColor");
	}
	
	public static float getUnitsPerLightYear() {
		return Global.getSettings().getFloat("unitsPerLightYear");
	}
	
	public static float getProfitMarginFlat() {
		return Global.getSettings().getFloat("profitMarginFlat");
	}
	
	public static float getProfitMarginMult() {
		return Global.getSettings().getFloat("profitMarginMult");
	}
	
	public static float getEconomyInterval() {
		return Global.getSettings().getFloat("economyIntervalnGameDays");
	}
	
	public static float getEconomyRollingAverageFactor() {
		return Global.getSettings().getFloat("economyRollingAverageFactor");
	}
	
	public static IntervalUtil createEconIntervalTracker() {
		float interval = getEconomyInterval();
		return new IntervalUtil(interval * 0.75f, interval * 1.25f);
	}
	
	public static String getAndJoined(String ... strings) {
		if (strings.length == 1) return strings[0];
		
		String result = "";
		for (int i = 0; i < strings.length - 1; i++) {
			result += strings[i] + ", ";
		}
		if (!result.isEmpty()) {
			result = result.substring(0, result.length() - 2);
		}
		if (strings.length >= 2) {
			result += " and " + strings[strings.length - 1];
		}
		return result;
	}
	
	public static interface FleetFilter {
		boolean accept(CampaignFleetAPI curr);
	}
	
	public static List<CampaignFleetAPI> findNearbyFleets(SectorEntityToken from, float maxRange, FleetFilter filter) {
		List<CampaignFleetAPI> result = new ArrayList<CampaignFleetAPI>();
		for (CampaignFleetAPI fleet : from.getContainingLocation().getFleets()) {
			if (fleet == from) continue;
			float dist = Misc.getDistance(fleet.getLocation(), from.getLocation());
			if (dist > maxRange) continue;
			
			if (filter == null || filter.accept(fleet)) {
				result.add(fleet);
			}
		}
		return result;
	}
	
	public static List<CampaignFleetAPI> getFleetsInOrNearSystem(StarSystemAPI system) {
		List<CampaignFleetAPI> result = new ArrayList<CampaignFleetAPI>(system.getFleets());
		for (CampaignFleetAPI fleet : Global.getSector().getHyperspace().getFleets()) {
			if (!fleet.isInOrNearSystem(system)) continue;
			result.add(fleet);
		}
		return result;
	}
	
	
	public static List<MarketAPI> getMarketsInLocation(LocationAPI location) {
		List<MarketAPI> result = new ArrayList<MarketAPI>();
		for (MarketAPI market : Global.getSector().getEconomy().getMarketsCopy()) {
			if (market.getContainingLocation() != location) continue;
			result.add(market);
		}
		return result;
	}
	
	public static List<MarketAPI> getNearbyMarkets(Vector2f locInHyper, float distLY) {
		List<MarketAPI> result = new ArrayList<MarketAPI>();
		for (MarketAPI market : Global.getSector().getEconomy().getMarketsCopy()) {
			float dist = getDistanceLY(market.getLocationInHyperspace(), locInHyper);
			if (dist > distLY) continue;
			result.add(market);
		}
		return result;
	}
	
	public static int getNumHostileMarkets(FactionAPI faction, SectorEntityToken from, float maxDist) {
		int hostileMarketsNearPoint = 0;
		for (MarketAPI market : Misc.getMarketsInLocation(from.getContainingLocation())) {
			SectorEntityToken primary = market.getPrimaryEntity();
			float dist = getDistance(primary.getLocation(), from.getLocation());
			if (dist > maxDist) continue;
			if (market.getFaction() != null && market.getFaction().isHostileTo(faction)) {
				hostileMarketsNearPoint ++;
			}
		}
		return hostileMarketsNearPoint;
	}
	
	public static StarSystemAPI getNearbyStarSystem(SectorEntityToken token, float maxRangeLY) {
		if (token.getContainingLocation() instanceof StarSystemAPI) {
			return (StarSystemAPI) token.getContainingLocation();
		}
		
		for (StarSystemAPI system : Global.getSector().getStarSystems()) {
			float dist = Misc.getDistanceLY(token.getLocationInHyperspace(), system.getLocation());
			if (dist > maxRangeLY) continue;
			return system;
		}
		return null;
	}
	
	public static StarSystemAPI getNearbyStarSystem(SectorEntityToken token) {
		if (token.getContainingLocation() instanceof StarSystemAPI) {
			return (StarSystemAPI) token.getContainingLocation();
		}
		for (StarSystemAPI system : Global.getSector().getStarSystems()) {
			if (token.isInOrNearSystem(system)) return system;
		}
		return null;
	}
	
	
	public static boolean showRuleDialog(SectorEntityToken entity, String initialTrigger) {
		RuleBasedInteractionDialogPluginImpl plugin;
		if (initialTrigger != null) {
			plugin = new RuleBasedInteractionDialogPluginImpl(initialTrigger);
		} else {
			plugin = new RuleBasedInteractionDialogPluginImpl();
		}
		return Global.getSector().getCampaignUI().showInteractionDialog(plugin, entity);
	}
	
	public static float getAngleInDegrees(Vector2f v) {
		Vector2f dir = new Vector2f(v);
		normalise(dir);
		float angle = (float) Math.toDegrees((float) Math.acos(dir.x));
		if (dir.y < 0) return 360 - angle;
		return angle;
	}
	public static Vector2f normalise(Vector2f v) {
		if (v.lengthSquared() > Float.MIN_VALUE) {
			return (Vector2f)v.normalise();
		}
		//return v;
		return new Vector2f(1, 0);
	}
	
	public static float normalizeAngle(float angleDeg) {
		return (angleDeg % 360f + 360f) % 360f;
	}
	
	public static MarketAPI findNearestLocalMarket(SectorEntityToken token, float maxDist, MarketFilter filter) {
		List<MarketAPI> localMarkets = getMarketsInLocation(token.getContainingLocation());
		float distToLocalMarket = Float.MAX_VALUE;
		MarketAPI closest = null;
		for (MarketAPI market : localMarkets) {
			if (filter != null && !filter.acceptMarket(market)) continue;
			
			if (market.getPrimaryEntity() == null) continue;
			if (market.getPrimaryEntity().getContainingLocation() != token.getContainingLocation()) continue;
			
			float currDist = Misc.getDistance(market.getPrimaryEntity().getLocation(), token.getLocation());
			if (currDist > maxDist) continue;
			if (currDist < distToLocalMarket) {
				distToLocalMarket = currDist;
				closest = market;
			}
		}
		return closest;
	}
	public static List<MarketAPI> findNearbyLocalMarkets(SectorEntityToken token, float maxDist, MarketFilter filter) {
		List<MarketAPI> localMarkets = getMarketsInLocation(token.getContainingLocation());
		List<MarketAPI> result = new ArrayList<MarketAPI>();
		
		for (MarketAPI market : localMarkets) {
			if (filter != null && !filter.acceptMarket(market)) continue;
			if (market.getPrimaryEntity() == null) continue;
			if (market.getPrimaryEntity().getContainingLocation() != token.getContainingLocation()) continue;
			
			float currDist = Misc.getDistance(market.getPrimaryEntity().getLocation(), token.getLocation());
			if (currDist > maxDist) continue;
			
			result.add(market);
			
		}
		return result;
	}
	
	public static MarketAPI findNearestLocalMarketWithSameFaction(final SectorEntityToken token, float maxDist) {
		return findNearestLocalMarket(token, maxDist, new MarketFilter() {
			public boolean acceptMarket(MarketAPI curr) {
				return curr.getFaction() == token.getFaction();
			}
		});
	}
	
	
	public static float RAD_PER_DEG = 0.01745329251f;
	public static Vector2f getUnitVectorAtDegreeAngle(float degrees) {
		Vector2f result = new Vector2f();
		float radians = degrees * RAD_PER_DEG;
		result.x = (float)Math.cos(radians);
		result.y = (float)Math.sin(radians);
		
		return result;
	}
	
	/**
	 * Angles.
	 * @param one
	 * @param two
	 * @param check
	 * @return
	 */
	public static boolean isBetween(float one, float two, float check) {
		one = normalizeAngle(one);
		two = normalizeAngle(two);
		check = normalizeAngle(check);

		//System.out.println(one + "," + two + "," + check);
		if (check >= one && check <= two) return true;
		
		if (one > two) {
			if (check <= two) return true;
			if (check >= one) return true;
		}
		return false;
	}
	
	public static float getShieldedCargoFraction(CampaignFleetAPI fleet) {
		float shielded = 0f;
		for (FleetMemberAPI member : fleet.getFleetData().getMembersListCopy()) {
			if (member.isMothballed()) continue;
			if (member.getVariant().hasHullMod(HullMods.SHIELDED_CARGO_HOLDS)) {
				shielded += member.getCargoCapacity();
			}
		}
		float max = fleet.getCargo().getMaxCapacity();
		if (max < 1) return 0f;
		return shielded / max;
	}
	
	
	public static Color interpolateColor(Color from, Color to, float progress) {
		float red = (float)from.getRed() + ((float)to.getRed() - (float)from.getRed()) * progress;
		float green = (float)from.getGreen() + ((float)to.getGreen() - (float)from.getGreen()) * progress;
		float blue = (float)from.getBlue() + ((float)to.getBlue() - (float)from.getBlue()) * progress;
		float alpha = (float)from.getAlpha() + ((float)to.getAlpha() - (float)from.getAlpha()) * progress;
		return new Color((int)red, (int)green, (int)blue, (int)alpha);
	}
	
	public static Color scaleColor(Color color, float factor) {
		return new Color((int) (color.getRed() * factor),
						 (int) (color.getGreen() * factor),
						 (int) (color.getBlue() * factor),
						 (int) (color.getAlpha() * factor));
	}
	public static Color scaleColorOnly(Color color, float factor) {
		return new Color((int) (color.getRed() * factor),
						 (int) (color.getGreen() * factor),
						 (int) (color.getBlue() * factor),
						 (int) (color.getAlpha()));
	}
	
	public static Color scaleAlpha(Color color, float factor) {
		return new Color((int) (color.getRed() * 1f),
				(int) (color.getGreen() * 1f),
				(int) (color.getBlue() * 1f),
				(int) (color.getAlpha() * factor));
	}
	
	public static Color setAlpha(Color color, int alpha) {
		if (alpha < 0) alpha = 0;
		if (alpha > 255) alpha = 255;
		return new Color(color.getRed(), color.getGreen(), color.getBlue(), alpha);
	}
	
	public static float getSizeNum(HullSize size) {
		switch (size) {
		case CAPITAL_SHIP:
			return 5;
		case CRUISER:
			return 3;
		case DESTROYER:
			return 2;
		case FIGHTER:
		case FRIGATE:
		case DEFAULT:
			return 1;
		}
		return 1;
	}
	
	public static void unsetAll(String prefix, String memKey, MemoryAPI memory) {
		Map<String, MemoryAPI> memoryMap = new HashMap<String, MemoryAPI>();
		memoryMap.put(memKey, memory);
		new unsetAll().execute(null, null, Misc.tokenize(prefix), memoryMap);
	}
	
	
	
	public static float getTargetingRadius(Vector2f from, CombatEntityAPI target, boolean considerShield) {
		return Global.getSettings().getTargetingRadius(from, target, considerShield);
	}
	
	
	public static float getClosingSpeed(Vector2f p1, Vector2f p2, Vector2f v1, Vector2f v2) {		
		// direction from target to shooter
		Vector2f dir = Vector2f.sub(p1, p2, new Vector2f());
		normalise(dir);
		// velocity of target relative to shooter
		Vector2f relVel = Vector2f.sub(v2, v1, new Vector2f());
		float closingSpeed = Vector2f.dot(dir, relVel);
		return closingSpeed;
	}
	
	/**
	 * DGS = digit group separator, i.e.: 1000000 -> 1,000,000
	 * @param num
	 * @return
	 */
	public static String getWithDGS(float num) {
		DecimalFormat format = new DecimalFormat("###,###,###,###,###");
		return format.format(num);
	}
	
	/**
	 * DGS = digit group separator, i.e.: 1000000 -> 1,000,000
	 * @param num
	 * @return
	 */
	public static String getDGSCredits(float num) {
		DecimalFormat format = new DecimalFormat("###,###,###,###,###");
		return format.format((int)num) + Strings.C;
	}
	
	public static float getAngleInDegrees(Vector2f from, Vector2f to) {
		Vector2f v = Vector2f.sub(to, from, new Vector2f());
		if (v.lengthSquared() == 0) return 0;
		return getAngleInDegrees(v);
	}
	
	public static Vector2f getInterceptPointBasic(SectorEntityToken from, SectorEntityToken to) {
		float dist = getDistance(from.getLocation(), to.getLocation()) - from.getRadius() - to.getRadius();
		if (dist <= 0) return new Vector2f(to.getLocation());
		
		float closingSpeed = getClosingSpeed(from.getLocation(), to.getLocation(), from.getVelocity(), to.getVelocity());
		if (closingSpeed <= 10) return new Vector2f(to.getLocation());
		
		Vector2f toTarget = getUnitVectorAtDegreeAngle(Misc.getAngleInDegrees(from.getLocation(), to.getLocation()));
		Vector2f vel = new Vector2f(from.getVelocity());
		normalise(vel);
		float dot = Vector2f.dot(toTarget, vel);
		if (dot < 0) return new Vector2f(to.getLocation());
//		if (to.isPlayerFleet()) {
//			System.out.println("23rwefe");
//		}
		float time = dist / closingSpeed;
		
		Vector2f point = new Vector2f(to.getVelocity());
		point.scale(time);
		Vector2f.add(point, to.getLocation(), point);
		return point;
	}
	
	
	
	/**
	 * A flag can be set to true for several "reasons". As long as it hasn't been set
	 * back to false for all of the "reasons", it will remain set to true.
	 * 
	 * For example, a fleet may be hostile because it's respond to comm relay interference,
	 * and because the player is running with the transponder off. Until both are resolved,
	 * the "hostile" flag will remain set to true.
	 * 
	 * Note: a flag can not be "set" to false. If it's set to false for all the current reasons,
	 * the key is removed from memory. 
	 * 
	 * Returns whether the flag is still set after this method does its work.
	 * @param memory
	 * @param flagKey
	 * @param reason
	 * @param value
	 * @return
	 */
	public static boolean setFlagWithReason(MemoryAPI memory, String flagKey, String reason, boolean value, float expire) {
		String requiredKey = flagKey + "_" + reason;

		if (value) {
			memory.set(flagKey, true);
			memory.set(requiredKey, value, expire);
			memory.addRequired(flagKey, requiredKey);
		} else {
			memory.unset(requiredKey);
		}
		
		return memory.contains(flagKey);
	}
	
	
	
	public static String getAgoStringForTimestamp(long timestamp) {
		CampaignClockAPI clock = Global.getSector().getClock();
		float days = clock.getElapsedDaysSince(timestamp);
		
		if (days <= 1f) {
			return "Today";
		} else if (days <= 6f) {
			return (int)Math.ceil(days) + " days ago";
		} else if (days <= 7) {
			return "1 week ago";
		} else if (days <= 14) {
			return "2 weeks ago";
		} else if (days <= 21) {
			return "3 weeks ago";
		} else if (days <= 30 + 14) {
			return "1 month ago";
		} else if (days < 30 * 2 + 14) {
			return "2 months ago";
		} else if (days < 30 * 3 + 14) {
			return "3 months ago";
		} else {
			return "Over 3 months ago";
		}
	}
	
	public static String getAtLeastStringForDays(int days) {
		if (days <= 1f) {
			return "at least a day";
		} else if (days <= 6f) {
			return "at least a few days";
		} else if (days <= 7 + 6) {
			return "at least a week";
		} else if (days <= 14 + 6) {
			return "at least 2 weeks";
		} else if (days <= 21 + 8) {
			return "at least 3 weeks";
		} else if (days <= 30 + 29) {
			return "at least a month";
		} else if (days < 30 * 2 + 29) {
			return "at least 2 months";
		} else if (days < 30 * 3 + 29) {
			return "at least 3 months";
		} else {
			return "many months";
		}
	}
	
//	public static String getTimeStringForDays(int days) {
//		if (days <= 1f) {
//			return "1 day";
//		} else if (days <= 6f) {
//			return "at least a few days";
//		} else if (days <= 7) {
//			return "at least a week";
//		} else if (days <= 14) {
//			return "at least 2 weeks";
//		} else if (days <= 21) {
//			return "at least 3 weeks";
//		} else if (days <= 30 + 14) {
//			return "at least a month";
//		} else if (days < 30 * 2 + 14) {
//			return "at least 2 months";
//		} else if (days < 30 * 3 + 14) {
//			return "at least 3 months";
//		} else {
//			return "many months";
//		}
//	}
	
	public static float getBurnLevelForSpeed(float speed) {
		speed -= Global.getSettings().getBaseTravelSpeed();
		if (speed < 0 || speed <= Global.getSettings().getFloat("minTravelSpeed") + 1f) speed = 0;
		float currBurn = speed / Global.getSettings().getSpeedPerBurnLevel();
		//System.out.println("ADFSDF: " +Math.round(currBurn));
		return Math.round(currBurn + 0.01f);
	}
	
	public static float getFractionalBurnLevelForSpeed(float speed) {
		speed -= Global.getSettings().getBaseTravelSpeed();
		if (speed < 0 || speed <= Global.getSettings().getFloat("minTravelSpeed") + 1f) speed = 0;
		float currBurn = speed / Global.getSettings().getSpeedPerBurnLevel();
		//System.out.println("ADFSDF: " +Math.round(currBurn));
		return currBurn;
	}
	
	public static float getSpeedForBurnLevel(float burnLevel) {
		float speed = Global.getSettings().getBaseTravelSpeed() + burnLevel * Global.getSettings().getSpeedPerBurnLevel();
		return speed;
	}
	
	public static float getFuelPerDay(CampaignFleetAPI fleet, float burnLevel) {
		float speed = Global.getSettings().getBaseTravelSpeed() + Global.getSettings().getSpeedPerBurnLevel() * burnLevel;
		return getFuelPerDayAtSpeed(fleet, speed);
	}
	
	public static float getFuelPerDayAtSpeed(CampaignFleetAPI fleet, float speed) {
		float perLY = fleet.getLogistics().getFuelCostPerLightYear();

		// this is potentially evil - currently, the velocity is in units per SECOND, not per day
		speed = speed * Global.getSector().getClock().getSecondsPerDay();
		// now, speed is in units per day
		
		speed = speed / Global.getSettings().getUnitsPerLightYear();
		// ly/day now
		
		return speed * perLY;
	}
	
	public static float getLYPerDayAtBurn(CampaignFleetAPI fleet, float burnLevel) {
		float speed = Global.getSettings().getBaseTravelSpeed() + Global.getSettings().getSpeedPerBurnLevel() * burnLevel;
		return getLYPerDayAtSpeed(fleet, speed);
	}
	public static float getLYPerDayAtSpeed(CampaignFleetAPI fleet, float speed) {
		// this is potentially evil - currently, the velocity is in units per SECOND, not per day
		speed = speed * Global.getSector().getClock().getSecondsPerDay();
		// now, speed is in units per day
		speed = speed / Global.getSettings().getUnitsPerLightYear();
		// ly/day now
		
		return speed * 1f; // 1f days
	}
	
	private static Vector3f temp4 = new Vector3f();
	public static float getDistance(Vector3f v1, Vector3f v2)
	{
		return Vector3f.sub(v1, v2, temp4).length();
	}
	
	public static float getAngleDiff(float from, float to) {
		float diff = normalizeAngle(from - to);
		if (diff > 180) return 360 - diff;
		else return diff;
	}
	
	public static boolean isInArc(float direction, float arc, Vector2f from, Vector2f to) {
		if (arc >= 360) return true;
		if (direction < 0) direction = 360 + direction;
		Vector2f towardsTo = new Vector2f(to.x - from.x, to.y - from.y);
		if (towardsTo.lengthSquared() == 0) return false;
		float dir = Misc.getAngleInDegrees(towardsTo);
		if (dir < 0) dir = 360 + dir;
		float arcFrom = direction - arc/2f;
		if (arcFrom < 0) arcFrom = 360 + arcFrom;
		if (arcFrom > 360) arcFrom -= 360;
		float arcTo = direction + arc/2f;
		if (arcTo < 0) arcTo = 360 + arcTo;
		if (arcTo > 360) arcTo -= 360;
		
		if (dir >= arcFrom && dir <= arcTo) return true;
		if (dir >= arcFrom && arcFrom > arcTo) return true;
		if (dir <= arcTo && arcFrom > arcTo) return true;
		return false;
	}
	
	public static boolean isInArc(float direction, float arc, float test) {
		test = normalizeAngle(test);
		
		if (arc >= 360) return true;
		if (direction < 0) direction = 360 + direction;
		float dir = test;
		if (dir < 0) dir = 360 + dir;
		float arcFrom = direction - arc/2f;
		if (arcFrom < 0) arcFrom = 360 + arcFrom;
		if (arcFrom > 360) arcFrom -= 360;
		float arcTo = direction + arc/2f;
		if (arcTo < 0) arcTo = 360 + arcTo;
		if (arcTo > 360) arcTo -= 360;
		
		if (dir >= arcFrom && dir <= arcTo) return true;
		if (dir >= arcFrom && arcFrom > arcTo) return true;
		if (dir <= arcTo && arcFrom > arcTo) return true;
		return false;
	}
	
	
	public static SectorEntityToken addNebulaFromPNG(String image, float centerX, float centerY, LocationAPI location,
			String category, String key, int tilesWide, int tilesHigh) {
		return addNebulaFromPNG(image, centerX, centerY, location, category, key, tilesWide, tilesHigh, Terrain.NEBULA);
	}
	
	
	public static SectorEntityToken addNebulaFromPNG(String image, float centerX, float centerY, LocationAPI location,
										String category, String key, int tilesWide, int tilesHigh,
										String terrainType) {
		try {
			BufferedImage img = null;
		    //img = ImageIO.read(new File("../starfarer.res/res/data/campaign/terrain/nebula_test.png"));
		    img = ImageIO.read(Global.getSettings().openStream(image));
		    
		    int chunkSize = 10000;
		    int w = img.getWidth();
		    int h = img.getHeight();
		    Raster data = img.getData();
		    for (int i = 0; i < w; i+= chunkSize) {
		    	for (int j = 0; j < h; j+= chunkSize) {
		    		
		    		int chunkWidth = chunkSize;
		    		if (i + chunkSize > w) chunkWidth = w - i;
		    		int chunkHeight = chunkSize;
		    		if (j + chunkSize > h) chunkHeight = h - i;
		    		
//		    		boolean hasAny = false;
//		    		for (int x = i; x < i + chunkWidth; x++) {
//		    			for (int y = j; y < j + chunkHeight; y++) {
//		    				int [] pixel = data.getPixel(i, h - j - 1, (int []) null);
//		    				int total = pixel[0] + pixel[1] + pixel[2];
//		    				if (total > 0) {
//		    					hasAny = true;
//		    					break;
//		    				}
//		    			}
//		    		}
//		    		if (!hasAny) continue;
		    		
		    		StringBuilder string = new StringBuilder();
		    		for (int y = j + chunkHeight - 1; y >= j; y--) {
		    			for (int x = i; x < i + chunkWidth; x++) {
		    				int [] pixel = data.getPixel(x, h - y - 1, (int []) null);
		    				int total = pixel[0] + pixel[1] + pixel[2];
		    				if (total > 0) {
		    					string.append("x");
		    				} else {
		    					string.append(" ");
		    				}
		    			}
		    		}
		    		
		    		float tileSize = NebulaTerrainPlugin.TILE_SIZE;
		    		float x = centerX - tileSize * (float) w / 2f + (float) i * tileSize + chunkWidth / 2f * tileSize;
		    		float y = centerY - tileSize * (float) h / 2f + (float) j * tileSize + chunkHeight / 2f * tileSize;
		    		
		    		SectorEntityToken curr = location.addTerrain(terrainType, new TileParams(string.toString(),
		    							chunkWidth, chunkHeight,
		    							category, key, tilesWide, tilesHigh));
		    		curr.getLocation().set(x, y);
		    		return curr;
		    	}
		    }
		    return null;
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	
	public static void renderQuad(float x, float y, float width, float height, Color color, float alphaMult) {
		GL11.glColor4ub((byte)color.getRed(),
						(byte)color.getGreen(),
						(byte)color.getBlue(),
						(byte)((float)color.getAlpha() * alphaMult));
		
		GL11.glBegin(GL11.GL_QUADS);
		{
			GL11.glVertex2f(x, y);
			GL11.glVertex2f(x, y + height);
			GL11.glVertex2f(x + width, y + height);
			GL11.glVertex2f(x + width, y);
		}
		GL11.glEnd();
	}
	
	/**
	 * Shortest distance from line to a point.
	 * @param p1 line1
	 * @param p2 line2
	 * @param p3 point
	 * @return
	 */
	public static float distanceFromLineToPoint(Vector2f p1, Vector2f p2, Vector2f p3) {
		float u = (p3.x - p1.x) * (p2.x - p1.x) + (p3.y - p1.y) * (p2.y - p1.y);
		float denom = Vector2f.sub(p2, p1, new Vector2f()).length();
		denom *= denom;
		//if (denom == 0) return 0;
		u /= denom;
		Vector2f i = new Vector2f();
		i.x = p1.x + u * (p2.x - p1.x);
		i.y = p1.y + u * (p2.y - p1.y);
		return Vector2f.sub(i, p3, new Vector2f()).length();
	}
	
	public static Vector2f closestPointOnLineToPoint(Vector2f p1, Vector2f p2, Vector2f p3) {
		float u = (p3.x - p1.x) * (p2.x - p1.x) + (p3.y - p1.y) * (p2.y - p1.y);
		float denom = Vector2f.sub(p2, p1, new Vector2f()).length();
		denom *= denom;
		//if (denom == 0) return 0;
		u /= denom;
		Vector2f i = new Vector2f();
		i.x = p1.x + u * (p2.x - p1.x);
		i.y = p1.y + u * (p2.y - p1.y);
		return i;
	}
	
	public static Vector2f closestPointOnSegmentToPoint(Vector2f p1, Vector2f p2, Vector2f p3) {
		float u = (p3.x - p1.x) * (p2.x - p1.x) + (p3.y - p1.y) * (p2.y - p1.y);
		float denom = Vector2f.sub(p2, p1, new Vector2f()).length();
		denom *= denom;
		
		u /= denom;
		
		// if closest point on line is outside the segment, clamp to on the segment
		if (u < 0) u = 0;
		if (u > 1) u = 1;
		
		Vector2f i = new Vector2f();
		i.x = p1.x + u * (p2.x - p1.x);
		i.y = p1.y + u * (p2.y - p1.y);
		return i;
	}
	
	public static boolean areSegmentsCoincident(Vector2f a1, Vector2f a2, Vector2f b1, Vector2f b2) {
		float denom = (b2.y - b1.y) * (a2.x - a1.x) - (b2.x - b1.x) * (a2.y - a1.y);
		float numUa = (b2.x - b1.x) * (a1.y - b1.y) - (b2.y - b1.y) * (a1.x - b1.x);
		float numUb = (a2.x - a1.x) * (a1.y - b1.y) - (a2.y - a1.y) *(a1.x - b1.x);

		if (denom == 0 && !(numUa == 0 && numUb == 0)) { // parallel, not coincident
			return false;
		}

		if (denom == 0 && numUa == 0 && numUb == 0) { // coincident
			return true;
		}
		
		return false;
	}
	
	public static Vector2f getPerp(Vector2f v) {
		Vector2f perp = new Vector2f();
		perp.x = v.y;
		perp.y = -v.x;
		return perp;
	}
	
	public static float getClosestTurnDirection(float facing, Vector2f from, Vector2f to) {
		Vector2f desired = getDiff(to, from);
		if (desired.lengthSquared() == 0) return 0;
		//float angle = getAngleInDegrees(desired);
		Vector2f more = getUnitVectorAtDegreeAngle(facing + 1);
		Vector2f less = getUnitVectorAtDegreeAngle(facing - 1);
		
		float fromMore = Vector2f.angle(more, desired);
		float fromLess = Vector2f.angle(less, desired);
		if (fromMore == fromLess) return 0f;
		if (fromMore > fromLess) return -1f;
		return 1f;
	}
	
	public static float getClosestTurnDirection(Vector2f one, Vector2f two) {
		return getClosestTurnDirection(getAngleInDegrees(one), new Vector2f(0, 0), two);
	}
	
	public static Vector2f getDiff(Vector2f v1, Vector2f v2)
	{
		//Vector2f result = new Vector2f();
		return Vector2f.sub(v1, v2, new Vector2f());
	}
	
	public static MarketAPI getSourceMarket(CampaignFleetAPI fleet) {
		String id = fleet.getMemoryWithoutUpdate().getString(MemFlags.MEMORY_KEY_SOURCE_MARKET);
		if (id == null) return null;
		MarketAPI market = Global.getSector().getEconomy().getMarket(id);
		return market;
	}
	
	public static float getSpawnChanceMult(Vector2f locInHyper) {
		if (Global.getSector().getPlayerFleet() == null) return 1f;
		
		float min = Global.getSettings().getFloat("minFleetSpawnChanceMult");
		float range = Global.getSettings().getFloat("minFleetSpawnChanceRangeLY");
		
		Vector2f playerLoc = Global.getSector().getPlayerFleet().getLocationInHyperspace();
		float distLY = getDistanceLY(playerLoc, locInHyper);
		
		float f = (1f - Math.min(1f, distLY/range));
		return min + (1f - min) * f * f;
	}
	
	public static Vector2f pickHyperLocationNotNearPlayer(Vector2f from, float minDist) {
		CampaignFleetAPI player = Global.getSector().getPlayerFleet();
		float r = 2000f;
		if (player == null || !player.isInHyperspace()) {
			return getPointWithinRadius(from, r);
		}
		float dist = Misc.getDistance(player.getLocation(), from);
		if (dist > minDist + r) {
			return getPointWithinRadius(from, r);
		}
		float dir = Misc.getAngleInDegrees(player.getLocation(), from);
		Vector2f v = Misc.getUnitVectorAtDegreeAngle(dir);
		v.scale(minDist + 2000 - dist);
		Vector2f.add(v, from, v);
		return getPointWithinRadius(v, r);
	}
	
	public static Vector2f pickLocationNotNearPlayer(LocationAPI where, Vector2f from, float minDist) {
		CampaignFleetAPI player = Global.getSector().getPlayerFleet();
		float r = 2000f;
		if (player == null || player.getContainingLocation() != where) {
			return getPointWithinRadius(from, r);
		}
		float dist = Misc.getDistance(player.getLocation(), from);
		if (dist > minDist + r) {
			return getPointWithinRadius(from, r);
		}
		float dir = Misc.getAngleInDegrees(player.getLocation(), from);
		Vector2f v = Misc.getUnitVectorAtDegreeAngle(dir);
		v.scale(minDist + 2000 - dist);
		Vector2f.add(v, from, v);
		return getPointWithinRadius(v, r);
	}
	
	public static float getBattleJoinRange() {
		return Global.getSettings().getFloat("battleJoinRange");
	}
	
	public static void wiggle(Vector2f v, float max) {
		v.x += (max * 2f * ((float) Math.random() - 0.5f));
		v.y += (max * 2f * ((float) Math.random() - 0.5f));
	}
	
	
	public static boolean isPlayerOrCombinedPlayerPrimary(CampaignFleetAPI fleet) {
		if (fleet.isPlayerFleet()) return true;
		
		if (fleet.getBattle() != null && fleet.getBattle().isOnPlayerSide(fleet) &&
				fleet.getBattle().isPlayerPrimary()) {
					return true;
		}
		return false;
	}
	
	public static boolean isPlayerOrCombinedContainingPlayer(CampaignFleetAPI fleet) {
		if (fleet.isPlayerFleet()) return true;
		
		if (fleet.getBattle() != null && fleet.getBattle().isOnPlayerSide(fleet)) {
			return true;
		}
		return false;
	}

	public static final String ASTEROID_SOURCE = "misc_astrdSource";
	
	public static AsteroidSource getAsteroidSource(SectorEntityToken asteroid) {
		if (asteroid.getCustomData().containsKey(ASTEROID_SOURCE)) {
			return (AsteroidSource) asteroid.getCustomData().get(ASTEROID_SOURCE);
		}
		return null;
	}
	public static void setAsteroidSource(SectorEntityToken asteroid, AsteroidSource source) {
		asteroid.getCustomData().put(ASTEROID_SOURCE, source);
	}
	public static void clearAsteroidSource(SectorEntityToken asteroid) {
		asteroid.getCustomData().remove(ASTEROID_SOURCE);
	}
	
	public static boolean isEasy() {
		return Difficulties.EASY.equals(Global.getSector().getDifficulty());
	}
	
	public static boolean isNormal() {
		return Difficulties.NORMAL.equals(Global.getSector().getDifficulty());
	}
	
	
	public static void main(String[] args) {
//		String s = "te\\\"st==$global.one $player.a $>=$sdfdsf\"++++dfdsf0---\"\"quoted \\\"=+!<>param\" one two 1234 1  2342 24  \t\t234234";
//		//String s = "menuState sdfsdf sdfsdf \"\"= \"main \\\" 1234\"";
//		//s = "!sdfsdAKJKFHD";
//		List<Token> result = tokenize(s);
//		for (Token str : result) {
//			System.out.println(str);
//		}
		//addNebulaFromPNG();
	}

}











