package org.dark.shaders;

import com.fs.starfarer.api.BaseModPlugin;
import com.fs.starfarer.api.Global;
import java.io.IOException;
import org.apache.log4j.Level;
import org.dark.shaders.distortion.DistortionShader;
import org.dark.shaders.light.LightData;
import org.dark.shaders.light.LightShader;
import org.dark.shaders.post.PostProcessShader;
import org.dark.shaders.util.ShaderLib;
import org.dark.shaders.util.TextureData;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * This is the base mod plugin for ShaderLib, which initializes its shaders.
 * <p>
 * @author DarkRevenant
 */
public final class ShaderModPlugin extends BaseModPlugin {

    private static final String SETTINGS_FILE = "shaderSettings.json";

    private static boolean useLargeRipple = false;
    private static boolean useSmallRipple = false;

    private static void loadSettings() throws IOException, JSONException {
        final JSONObject settings = Global.getSettings().loadJSON(SETTINGS_FILE);

        final boolean enabled = settings.getBoolean("enableDistortion");
        if (enabled) {
            useLargeRipple = settings.getBoolean("useLargeRipple");
            if (!useLargeRipple) {
                useSmallRipple = true;
            }
        }
    }

    @Override
    public void onApplicationLoad() throws IOException {
        Global.getLogger(ShaderModPlugin.class).setLevel(Level.ERROR);

        ShaderLib.init();

        if (ShaderLib.areShadersAllowed()) {
            //ShaderLib.addShaderAPI(new LensShader());
            //ShaderLib.addShaderAPI(new InvertShader());
            ShaderLib.addShaderAPI(new PostProcessShader());
        }

        if (ShaderLib.areShadersAllowed() && ShaderLib.areBuffersAllowed()) {
            LightData.readLightDataCSVNoOverwrite("data/lights/core_light_data.csv");
            TextureData.readTextureDataCSVNoOverwrite("data/lights/core_texture_data.csv");
            ShaderLib.addShaderAPI(new LightShader());
            ShaderLib.addShaderAPI(new DistortionShader());

            try {
                loadSettings();
            } catch (IOException | JSONException e) {
                Global.getLogger(ShaderModPlugin.class).log(Level.ERROR, "Failed to load shader settings: " + e.getMessage());
            }

            if (useSmallRipple || useLargeRipple) {
                String path = "graphics/shaders/distortions/wave.png";
                try {
                    Global.getSettings().loadTexture(path);
                } catch (IOException e) {
                    Global.getLogger(ShaderModPlugin.class).log(Level.ERROR, "Texture loading failed at " + path + "! " + e.getMessage());
                    throw e; // Crash the game; it's probably too fucked to work at this point, anyway
                }
            }

            if (useSmallRipple) {
                String path = "";
                try {
                    for (int i = 1; i <= 60; i++) {
                        if (i < 10) {
                            path = "graphics/shaders/distortions/smallripple/000" + i + ".PNG";
                        } else {
                            path = "graphics/shaders/distortions/smallripple/00" + i + ".PNG";
                        }
                        Global.getSettings().loadTexture(path);
                    }
                } catch (IOException e) {
                    Global.getLogger(ShaderModPlugin.class).log(Level.ERROR, "Texture loading failed at " + path + "! " + e.getMessage());
                    throw e; // Crash the game; it's probably too fucked to work at this point, anyway
                }
            } else if (useLargeRipple) {
                String path = "";
                try {
                    for (int i = 1; i <= 60; i++) {
                        if (i < 10) {
                            path = "graphics/shaders/distortions/ripple/000" + i + ".PNG";
                        } else {
                            path = "graphics/shaders/distortions/ripple/00" + i + ".PNG";
                        }
                        Global.getSettings().loadTexture(path);
                    }
                } catch (IOException e) {
                    Global.getLogger(ShaderModPlugin.class).log(Level.ERROR, "Texture loading failed at " + path + "! " + e.getMessage());
                    throw e; // Crash the game; it's probably too fucked to work at this point, anyway
                }
            }
        }
    }
}
